/*
*  allpingersdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "allpingersdlgs.h"
#include "statsdlgs.h"
#include "proxydlgs.h"
#include "utility.h"
#include "qsdbs.h"
#include "mixmaster.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TAllPingersDlg *AllPingersDlg;
const AnsiString firstallpingersmsg =
"    Before I can update remailer statistics or send your mixmaster "
"message, I need to go online and get a document called allpingers.txt. That doc lists "
"all the sources for stats and keys and the urls I need in order to get them.\n\nIf you "
"need to assign an HTTP proxy do that now. It'll be used for all HTTP accesses.\n\n"
"Click [Ok] to get allpingers.txt.";
//---------------------------------------------------------------------------

bool __fastcall CheckGetFirstAllPingers(TForm *Owner)
{
   TAllPingersDlg *allpingersdlg = new TAllPingersDlg(Owner);
   allpingersdlg->ShowModal();
   bool ok = allpingersdlg->ModalResult == mrOk;
   delete allpingersdlg;
   if (!ok)
      return false;

   if (AutoUpdateAllPingers(Owner))
   {
      Snooze(300);
      MsgBox(Owner, "Ok, I got it! Thanks! ", MT_INFO, MB_OK);
      Snooze(300);
   }
   return FileExists(MixAllPingersFile());
}
//---------------------------------------------------------------------------

__fastcall TAllPingersDlg::TAllPingersDlg(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Allpingers->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   delete ini;
   ResetBtn->Height = Allpingers->Height - 1;

   RichEdit1->Lines->Text = firstallpingersmsg;
   config = new TStringList();
   DB->LoadConfig("http", config);
   Allpingers->Text = ReadString(config, mixallpingerstxt, MixAllPingersURL);
   assert(!MixAllPingersURL.IsEmpty());
}
//---------------------------------------------------------------------------

void __fastcall TAllPingersDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (Allpingers->Text.LowerCase().Pos("http://") != 1)
      {
         MsgBox(this,
            "Problem! This operation requires a URL in the \n"
            "Allpingers.txt URL field.",
             MT_INFO, MB_OK);
         CanClose = false;
         return;
      }

      if (Allpingers->Text != config->Values[mixallpingerstxt])
      {
         config->Values[mixallpingerstxt] = Allpingers->Text;
         DB->SaveConfig("http", config);
      }
   }
   delete config;
}
//---------------------------------------------------------------------------

void __fastcall TAllPingersDlg::ResetBtnClick(TObject *Sender)
{
   Allpingers->Text = MixAllPingersURL;
   Allpingers->Modified = true;
}
//---------------------------------------------------------------------------

void __fastcall TAllPingersDlg::ProxyBtnClick(TObject *Sender)
{
   //EditHTTPProxy(this);
}
//---------------------------------------------------------------------------
