/*
*  contactdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <clipbrd.hpp>

#include "contactdlgs.h"
#include "global.h"
#include "utility.h"
#include "qsdbs.h"
#include "msgmans.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TContactDlg *ContactDlg;
//---------------------------------------------------------------------------

AnsiString __fastcall NewContactProfile(TForm *Owner)
{
   AnsiString email;
   TContactDlg *contact = new TContactDlg(Owner, "");
   contact->ShowModal();
   bool ok = contact->ModalResult == mrOk;
   if (ok)
      email = contact->Email->Text;
   delete contact;
   return email;
}
//---------------------------------------------------------------------------

AnsiString __fastcall EditContactProfile(TForm *Owner, const AnsiString &Email)
{
   AnsiString email;
   TContactDlg *contact = new TContactDlg(Owner, Email);
   contact->ShowModal();
   bool ok = contact->ModalResult == mrOk;
   if (ok)
      email = contact->Email->Text;
   delete contact;
   return email;
}
//---------------------------------------------------------------------------

bool __fastcall AddContactFromMessage(TForm *Owner, const AnsiString &Email)
{
   bool exists = false;
   if (DB->ContactExists(Email))
   {
	  if (MsgBox(Owner,
		 "This contact is in already in your contact list! Would you    \n"
		 "like to open that contact anyway?   ",
		  MT_INFO, MB_OKCANCEL) == IDCANCEL)
	  {
		 return false;
	  }
	  exists = true;
   }
   TContactDlg *contact = new TContactDlg(Owner, Email);
   contact->Email->Modified = !exists;
   contact->ActiveControl = contact->Notes;
   contact->ShowModal();
   bool ok = contact->ModalResult == mrOk;
   delete contact;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TContactDlg::TContactDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TContactDlg::TContactDlg(TComponent* Owner, const AnsiString &email)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "contactdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Email->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   AssignFont(Notes->Font, ini->ReadString("settings", "editorfont",
      OptsDefEditorFont));
   delete ini;
   Notes->Clear();

   orgemail = email;
   Email->Text = orgemail;
   if (!email.IsEmpty())
      DB->LoadContact(orgemail, Notes->Lines);
   Notes->SelStart = 0;
   Notes->Perform(EM_SCROLLCARET, 0, 0);
   Email->Modified = false;
   Notes->Modified = false;
}
//---------------------------------------------------------------------------

__fastcall TContactDlg::~TContactDlg()
{
   if (IntToStr(Width)+IntToStr(Height) != orgwin)
   {
      TIniFile *ini = new TIniFile(AppIni());
      SaveWindowPosition(this, "contactdlg", ini);
      delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TContactDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (Email->Text.IsEmpty())
      {
         MsgBox(this,
            "Problem! The 'Email' field can't be empty.  ",
             MT_INFO, MB_OK);
         CanClose = false;
         return;
      }
      TMsgManager *man = new TMsgManager(NULL);
      bool ok = man->ValidAddress(man->GetAddress(Email->Text));
      delete man;
      if (!ok)
      {
         MsgBox(this,
            "Problem! You email address isn't a valid address!   ",
             MT_INFO, MB_OK);
         CanClose = false;
         return;
      }
   }

   if (Email->Modified || Notes->Modified)
   {
      if (ModalResult == mrCancel)
      {
         switch(MsgBox(this,
            "Your contact info is modified. Save your changes?   ",
             MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      if (!orgemail.IsEmpty() && Email->Text != orgemail)
         DB->DelContact(orgemail);

      TStringList *notes = new TStringList();
      GetUnwrappedText(Notes, notes);
      DB->SaveContact(Email->Text, notes);
      delete notes;
   }
}
//---------------------------------------------------------------------------

void __fastcall TContactDlg::Cut1Click(TObject *Sender)
{
   Notes->CutToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TContactDlg::Copy1Click(TObject *Sender)
{
   Notes->CopyToClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TContactDlg::Paste1Click(TObject *Sender)
{
   Notes->PasteFromClipboard();
}
//---------------------------------------------------------------------------

void __fastcall TContactDlg::Selectall1Click(TObject *Sender)
{
   Notes->SelectAll();
}
//---------------------------------------------------------------------------

void __fastcall TContactDlg::PopupMenu1Popup(TObject *Sender)
{
   bool sel = Notes->SelLength > 0;
   Cut1->Enabled = sel;
   Copy1->Enabled = sel;
   Paste1->Enabled = Clipboard()->HasFormat(CF_TEXT);
   Selectall1->Enabled = Notes->Text.Length() > 0;
}
//---------------------------------------------------------------------------

