/*
*  eheadersdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "eheadersdlgs.h"
#include "qsdbs.h"
#include "utility.h"
#include "global.h"
#include "inputdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//TEHeadersDlg *EHeadersDlg;
const AnsiString dlgmsg =
"This is a simple spell check mechanism on the headers in your message. "
"To use a header, it must be present in this list. If you are compelled "
"to use a header not listed here then you can add it--taking great care in "
"spelling and format. Remailers strip many headers so don't be surprised if "
"yours disappears.";
const AnsiString DefValidHeaders =
"Attachment:,Bcc:,Cc:,Chain:,Content-Type:,Date:,Fcc:,From:,Host:,MIME-Version:,"
"Newsgroups:,Organization:,Pgp:,Post:,References:,Sign-On:,Subject:,To:,"
"X-No-Archive:,Proxy:,Nym:";
//---------------------------------------------------------------------------

bool __fastcall EditEmailHeaders(TForm *Owner)
{
   TEHeadersDlg *eheadersdlg = new TEHeadersDlg(Owner);
   eheadersdlg->ShowModal();
   bool ok = eheadersdlg->saved;
   delete eheadersdlg;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall LoadAcceptedHeaders(TStrings *hdrs)
{

   DB->LoadConfig(EmailHdrs, hdrs);
   if (!hdrs->Count)
      hdrs->CommaText = DefValidHeaders;
}
//---------------------------------------------------------------------------

__fastcall TEHeadersDlg::TEHeadersDlg(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "ehdrsdlg", ini);
   orgwin = IntToStr(Width)+IntToStr(Height);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(HeaderList->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   delete ini;

   RichEdit1->Lines->Text = dlgmsg;
   //RichEdit1->SelStart = 0;
   //RichEdit1->Perform(EM_SCROLLCARET, 0, 0);
   LoadAcceptedHeaders(HeaderList->Items);
   orghdrs = HeaderList->Items->CommaText;
}
//---------------------------------------------------------------------------

__fastcall TEHeadersDlg::~TEHeadersDlg()
{
   if (IntToStr(Width)+IntToStr(Height) != orgwin)
   {
      TIniFile *ini = new TIniFile(AppIni());
      SaveWindowPosition(this, "ehdrsdlg", ini);
      delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   if (HeaderList->Items->CommaText != orghdrs)
   {
      if (ModalResult == mrCancel)
      {
         switch(MsgBox(this,
            "Your list is modified. Save your changes? ",
             MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      DB->SaveConfig(EmailHdrs, HeaderList->Items);
    //DB->PutText(EmailHdrs, HeaderList->Items);
      saved = true;
   }
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::PopupResetClick(TObject *Sender)
{
   if (MsgBox(this,
      "Reset the header list to it's original state? ",
       MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
      HeaderList->Clear();
      Snooze(200);
      HeaderList->Items->CommaText = DefValidHeaders;
   }
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::DeleteBtnClick(TObject *Sender)
{
   int x = HeaderList->ItemIndex;
   assert(x >= 0);
   if (MsgBox(this,
      "Delete the header, " + HeaderList->Items->Strings[x] + " ? ",
       MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  HeaderList->Items->Delete(x);
	  HeaderList->ItemIndex = -1;
	  DeleteBtn->Enabled = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::AddBtnClick(TObject *Sender)
{
   TStringList *hdrs = new TStringList();
   hdrs->CaseSensitive = false;
   hdrs->AddStrings(HeaderList->Items);
   AnsiString hdr = GetUserInput(this, "New Header", "&Enter header to add", "", hdrs);
   delete hdrs;
   if (!hdr.IsEmpty())
   {
      hdr = hdr.Trim();
      if (hdr.Pos(":") != hdr.Length())
         hdr += ":";
      HeaderList->Items->Add(hdr);
   }
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::PopupMenuPopup(TObject *Sender)
{
   PopupDelete->Enabled = HeaderList->ItemIndex >= 0;
   PopupResizedialog->Enabled = BorderStyle == bsDialog;
   PopupLocksize->Enabled = BorderStyle == bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::PopupResizedialogClick(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::PopupLocksizeClick(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::FormResize(TObject *Sender)
{
   TRect Rct = Rect(0, 0, 0, 0);
   RichEdit1->Perform(EM_SETRECT,0,long(&Rct));
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::HeaderListDrawItem(TWinControl *Control,
	  int Index, TRect &Rect, TOwnerDrawState State)
{
   HeaderList->Canvas->TextRect(Rect, Rect.Left + 5, Rect.Top - 1,
	  HeaderList->Items->Strings[Index]);
}
//---------------------------------------------------------------------------

void __fastcall TEHeadersDlg::HeaderListClick(TObject *Sender)
{
   DeleteBtn->Enabled = HeaderList->ItemIndex >= 0;
}
//---------------------------------------------------------------------------

