/*
*  eraser.h 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#ifndef eraserH
#define eraserH

#include <vcl\forms.hpp>
#include <vcl\Classes.hpp>
//---------------------------------------------------------------------------

bool __fastcall InitEraserLibrary(); // called at program startup
void __fastcall FreeEraserLibrary(); // called at program end

AnsiString __fastcall EraserDir();
AnsiString __fastcall EraserDll();
AnsiString __fastcall EraserVersion();

typedef ULONG E_UINT32, *E_PUINT32;
typedef E_UINT32 ERASER_HANDLE;

bool __fastcall eraserWipeFile(ERASER_HANDLE &context, const AnsiString &FileName,
   bool newthread = false);
bool __fastcall eraserWipeFile(const AnsiString &filename, bool newthread = false);
bool __fastcall eraserWipeFiles(ERASER_HANDLE context, TStrings *filenames,
   bool newthread = false);
bool __fastcall eraserWipeFiles(TStrings *filenames, bool newthread = false);
void __fastcall eraserClearDirectory(ERASER_HANDLE context,
   const AnsiString &directory, bool newthread = false);
void __fastcall eraserClearDirectory(const AnsiString &directory,
   bool newthread = false);

typedef char        E_INT8,   *E_PINT8;
typedef short       E_INT16,  *E_PINT16;
typedef LONG        E_INT32,  *E_PINT32;
typedef LONGLONG    E_INT64,  *E_PINT64;

typedef BYTE        E_UINT8,  *E_PUINT8;
typedef WORD        E_UINT16, *E_PUINT16;
typedef ULONGLONG   E_UINT64, *E_PUINT64;

typedef E_INT32 ERASER_RESULT;
typedef enum {
    ERASER_METHOD_LIBRARY,
    ERASER_METHOD_GUTMANN,
    ERASER_METHOD_DOD,
    ERASER_METHOD_DOD_E,
    ERASER_METHOD_PSEUDORANDOM } ERASER_METHOD;

typedef enum {
    ERASER_DATA_DRIVES,
    ERASER_DATA_FILES } ERASER_DATA_TYPE;

#define E_IN        const
#define E_OUT
#define E_INOUT

#define ERASER_OK                       0       // No error
#define ERASER_ERROR                    -1      // Unspecified error
#define ERASER_ERROR_PARAM1             -2      // Parameter 1 invalid
#define ERASER_ERROR_PARAM2             -3      // Parameter 2 invalid
#define ERASER_ERROR_PARAM3             -4      // Parameter 3 invalid
#define ERASER_ERROR_PARAM4             -5      // Parameter 4 invalid
#define ERASER_ERROR_PARAM5             -6      // Parameter 5 invalid
#define ERASER_ERROR_PARAM6             -7      // Parameter 6 invalid
#define ERASER_ERROR_MEMORY             -8      // Out of memory
#define ERASER_ERROR_THREAD             -9      // Failed to start thread
#define ERASER_ERROR_EXCEPTION          -10     // BUG!
#define ERASER_ERROR_CONTEXT            -11     // Context array full (ERASER_MAX_CONTEXT)
#define ERASER_ERROR_INIT               -12     // Library not initialized (eraserInit())
#define ERASER_ERROR_RUNNING            -13     // Failed because the thread is running
#define ERASER_ERROR_NOTRUNNING         -14     // Failed because the thread is not running
#define ERASER_ERROR_DENIED             -15     // Operation not permitted
#define ERASER_ERROR_NOTIMPLEMENTED     -32     // Function not implemented

#define eraserOK(x)     ((x) >= ERASER_OK)
#define eraserError(x)  (!eraserOK(x))

#define ERASER_EXPORT __declspec(dllexport) ERASER_RESULT __stdcall

ERASER_EXPORT eraserInit();
ERASER_EXPORT eraserEnd();
ERASER_EXPORT eraserCreateContextEx(E_OUT ERASER_HANDLE*, E_IN E_UINT16, E_IN E_UINT16, E_IN E_UINT8);
ERASER_EXPORT eraserDestroyContext(E_IN ERASER_HANDLE);
ERASER_EXPORT eraserSetDataType(E_IN ERASER_HANDLE, E_IN E_UINT16);
ERASER_EXPORT eraserAddItem(E_IN ERASER_HANDLE, E_IN LPVOID, E_IN E_UINT16);
ERASER_EXPORT eraserStartSync(E_IN ERASER_HANDLE);
ERASER_EXPORT eraserStart(E_IN ERASER_HANDLE);
ERASER_EXPORT eraserStatGetArea(E_IN ERASER_HANDLE, E_OUT E_PUINT64);
ERASER_EXPORT eraserShowReport(E_IN ERASER_HANDLE, E_IN HWND);
ERASER_EXPORT eraserRemoveFile(E_IN LPVOID, E_IN E_UINT16);
ERASER_EXPORT eraserErrorStringCount(E_IN ERASER_HANDLE, E_OUT E_PUINT16);
ERASER_EXPORT eraserErrorString(E_IN ERASER_HANDLE, E_IN E_UINT16, E_OUT LPVOID, E_INOUT E_PUINT16);
ERASER_EXPORT eraserIsValidContext(E_IN ERASER_HANDLE);
ERASER_EXPORT eraserProgGetTotalPercent(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
ERASER_EXPORT eraserCompleted(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
ERASER_EXPORT eraserFailed(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
ERASER_EXPORT eraserStop(E_IN ERASER_HANDLE);
ERASER_EXPORT eraserIsRunning(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
AnsiString eraserVersion();

typedef ERASER_EXPORT (*ERASERINIT)();
typedef ERASER_EXPORT (*ERASEREND)();
typedef ERASER_EXPORT (*ERASERCREATECONTEXTEX)(E_OUT ERASER_HANDLE*, E_IN E_UINT16, E_IN E_UINT16, E_IN E_UINT8);
typedef ERASER_EXPORT (*ERASERISVALIDCONTEXT)(E_IN ERASER_HANDLE);
typedef ERASER_EXPORT (*ERASERDESTROYCONTEXT)(E_IN ERASER_HANDLE);
typedef ERASER_EXPORT (*ERASERSETDATATYPE)(E_IN ERASER_HANDLE, E_IN E_UINT16);
typedef ERASER_EXPORT (*ERASERADDITEM)(E_IN ERASER_HANDLE, E_IN LPVOID, E_IN E_UINT16);
typedef ERASER_EXPORT (*ERASERSTARTSYNC)(E_IN ERASER_HANDLE);
typedef ERASER_EXPORT (*ERASERSTART)(E_IN ERASER_HANDLE);
typedef ERASER_EXPORT (*ERASERSTATGETAREA)(E_IN ERASER_HANDLE, E_OUT E_PUINT64);
typedef ERASER_EXPORT (*ERASERSHOWREPORT)(E_IN ERASER_HANDLE, E_IN HWND);
typedef ERASER_EXPORT (*ERASERREMOVEFILE)(E_IN LPVOID, E_IN E_UINT16);
typedef ERASER_EXPORT (*ERASERERRORSTRINGCOUNT)(E_IN ERASER_HANDLE, E_OUT E_PUINT16);
typedef ERASER_EXPORT (*ERASERERRORSTRING)(E_IN ERASER_HANDLE, E_IN E_UINT16, E_OUT LPVOID, E_INOUT E_PUINT16);
typedef ERASER_EXPORT (*ERASERPROGGETTOTALPERCENT)(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
typedef ERASER_EXPORT (*ERASERCOMPLETED)(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
typedef ERASER_EXPORT (*ERASERFAILED)(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
typedef ERASER_EXPORT (*ERASERSTOP)(E_IN ERASER_HANDLE);
typedef ERASER_EXPORT (*ERASERISRUNNING)(E_IN ERASER_HANDLE, E_OUT E_PUINT8);
typedef LPCTSTR       (*ERASERVERSION)();

AnsiString __fastcall EraserErrorString(int error);

const AnsiString ERASERenabled = "eraser";
const bool ERASEREnabled = true;
const AnsiString ERASERwipemethod = "method";
const AnsiString ERASERWipeMethod = "dod";
const AnsiString ERASERpasses = "passes";
const int ERASERPasses = 1;
//---------------------------------------------------------------------------
#endif
