/*
*  httpmod.cpp 
*
*  Copyright (C) 2010 Richard Christman
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by the
*  Free Software Foundation; either version 3 of the License, or (at your
*  option) any later version.
*
*  This program is distributed in the hope that it will be useful, but
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*  for more details.
*
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
*
*  Additional permission under GNU GPL version 3 section 7
*
*  If you modify QuickSilver Lite, or any covered work, by linking or
*  combining it with CodeGear VCL (or a modified version of that library),
*  containing parts covered by the terms of CodeGear's license, the licensors
*  of QuickSilver Lite grant you additional permission to convey the
*  resulting work.
*
*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "httpmod.h"
#include "utility.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

bool __fastcall HTTPGetURL(TForm *Owner, const AnsiString &URL, TStrings *Proxy,
   TStrings *Log, TStrings *Page, AnsiString &ErrorMsg, bool *Cancel)
{
	assert(URL != "");
	assert(Log != NULL);
	assert(Page != NULL);
	assert(Cancel != NULL);

	TQSHTTPThread *pager = new TQSHTTPThread(Owner, URL, Proxy, Log, Page, Cancel);
	while (!pager->finished)
	{
	   Application->ProcessMessages();
	   Sleep(100);
	}
	bool ok = pager->success;
	if (!ok) {
	   ErrorMsg = pager->errormsg;
	}
	delete pager;
	return ok;
}

//---------------------------------------------------------------------------
// TQSHTTPThread
//---------------------------------------------------------------------------

__fastcall TQSHTTPThread::TQSHTTPThread(bool CreateSuspended)
   : TThread(CreateSuspended)
{
}
//---------------------------------------------------------------------------

__fastcall TQSHTTPThread::TQSHTTPThread(TForm *owner, const AnsiString &URL,
   TStrings *Proxy, TStrings *Log, TStrings *Page, bool *Cancel)
   : TThread(false)
{
   log = Log;
   Owner = owner;
   success = true;
   finished = false;
   unsigned short port;
   FreeOnTerminate = false;
   page = Page;
   TheURL = URL;
   errormsg = "";
   socksinfo = NULL;
   HTTP = new TQSHTTP(Owner);
   HTTP->Intercept = new TQSLogEvent(HTTP);
  ((TQSLogEvent*)HTTP->Intercept)->thread = this;
  ((TQSLogEvent*)HTTP->Intercept)->http = HTTP;
   HTTP->cancel = Cancel;
   HTTP->log = Log;
   ssliohandler = new TIdSSLIOHandlerSocketOpenSSL(HTTP);
   ssliohandler->SSLOptions->Method = sslvTLSv1;
   ssliohandler->SSLOptions->Mode = sslmClient;
   ssliohandler->SSLOptions->VerifyMode.Clear();
   ssliohandler->SSLOptions->VerifyDepth = 0;
   ssliohandler->SSLOptions->SSLVersions = httpsslversions;
   ssliohandler->SSLOptions->CipherList = httpciphersuite;
   HTTP->IOHandler = ssliohandler;
   HTTP->IOHandler->Intercept = HTTP->Intercept;

   // setup whatever proxy
   HTTP->ProxyParams->ProxyPort = 0;
   HTTP->ProxyParams->ProxyServer = "";
   HTTP->ProxyParams->BasicAuthentication = false;
   HTTP->ProxyParams->ProxyPassword = "";
   HTTP->ProxyParams->ProxyUsername = "";

   if (Proxy && Proxy->Count) {
	  AnsiString socks = Proxy->Values["socks"].LowerCase();
	  if (socks.IsEmpty() || socks == "no" || socks == "none") {
		 // http proxy
		 HTTP->ProxyParams->ProxyPort = Proxy->Values["port"].ToIntDef(80);
		 HTTP->ProxyParams->ProxyServer = Proxy->Values["host"];
		 if (Proxy->Values["auth"] == "1") {
			HTTP->ProxyParams->BasicAuthentication = true;
			HTTP->ProxyParams->ProxyPassword = Proxy->Values["pswd"];
			HTTP->ProxyParams->ProxyUsername = Proxy->Values["user"];
		 }
	  }
	  else
	  {  // socks proxy
		 if (socks != "4" && socks != "4a" && socks != "5" && socks != "none" &&
			socks != "no" && socks != "")
		 {
			errormsg =
			   "FATAL ERROR: Your proxy definition has an invalid socks\n"
			   "version '"+socks+"'. It must be 4, 4a, 5 or none.";
			success = false;
			finished = true;
			return;
		 }

		 socksinfo = new TIdSocksInfo(HTTP);
		 socksinfo->Port = 0;
		 socksinfo->Host = "";
		 socksinfo->Authentication = saNoAuthentication;
		 socksinfo->Username = "";
		 socksinfo->Password = "";

		 if (socks == "4a")     socksinfo->Version = svSocks4A;
		 else if (socks == "4") socksinfo->Version = svSocks4;
		 else if (socks == "5") socksinfo->Version = svSocks5;

		 socksinfo->Host = Proxy->Values["host"];
		 socksinfo->Port = Proxy->Values["port"].ToIntDef(80);
		 if (Proxy->Values["auth"] == "1") {
			socksinfo->Authentication = saUsernamePassword;
			socksinfo->Username = Proxy->Values["user"];
			socksinfo->Password = Proxy->Values["pswd"];
		 }
		 ssliohandler->TransparentProxy = socksinfo;
	  }
   }
}
//---------------------------------------------------------------------------

__fastcall TQSHTTPThread::~TQSHTTPThread()
{
   if (socksinfo)
	  delete socksinfo;
   delete ssliohandler;
   delete HTTP->IOHandler;
   delete HTTP->Intercept;
   delete HTTP;
}
//---------------------------------------------------------------------------

AnsiString __fastcall striplf(AnsiString msg)
{
   int p;
   while ((p = msg.Pos('\r')) > 0)
	  msg[p] = ' ';
   while ((p = msg.Pos('\n')) > 0)
	  msg[p] = ' ';
   return msg;
}
//---------------------------------------------------------------------------

void __fastcall TQSHTTPThread::Execute()
{
   success = false;
   finished = false;
   try
   {
	  HTTP->Get(TheURL, HTTP->Content);
   }
   catch (EAbort &e)
   {
	  errormsg = "operation canceled";
	  finished = true;
	  return;
   }
   catch (EIdHTTPProtocolException &e)
   {
	  errormsg = "Error: "+AnsiString(e.ErrorCode)+" "+e.Message;
	  finished = true;
	  return;
   }
   catch (EIdConnClosedGracefully &e)
   {
	  errormsg = "server closed connection";
	  finished = true;
   }
   catch (EIdUnknownProtocol &e)
   {
	  errormsg = "Error: "+e.Message;
	  finished = true;
	  return;
   }
   catch (EIdSocketError &e)
   {
	  errormsg = "Error: "+striplf(e.Message);
	  finished = true;
	  return;
   }
   catch (EIdException &e)
   {
	  errormsg = "Error: "+e.Message;
	  finished = true;
	  return;
   }
   catch (Exception &e)
   {
	  errormsg = "Error: "+e.Message;
	  finished = true;
	  return;
   }
   catch(...)
   {
	  errormsg = "Something wicked happened!";
	  finished = true;
	  return;
   }
   finished = true;
   success = true;
}
//---------------------------------------------------------------------------

void __fastcall TQSHTTPThread::Log(AnsiString line)
{
   log->Add(line);
}

//---------------------------------------------------------------------------
// TQSLogEvent
//---------------------------------------------------------------------------

__fastcall TQSLogEvent::TQSLogEvent(TComponent *owner) :
   TIdLogEvent(owner)
{
   Owner = (TForm*)Owner;
   OnConnect = LogEventConnect;
   OnDisconnect = LogEventDisconnect;
   OnReceive = LogEventReceive;
   OnReceived = LogEventReceived;
   OnSend = LogEventSend;
   OnSent = LogEventSent;
   OnStatus = LogEventStatus;
   ReplaceCRLF = false;
   Active = true;
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventStatus(TComponent *ASender, const AnsiString AText)
{
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventConnect(TIdConnectionIntercept *ASender)
{
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventSend(TIdConnectionIntercept *ASender, TIdBytes &ABuffer)
{
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventSent(TComponent *ASender, const AnsiString AText, const AnsiString AData)
{
   thread->Log("> "+AData);
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventReceive(TIdConnectionIntercept *ASender, TIdBytes &ABuffer)
{
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventReceived(TComponent *ASender, const AnsiString AText, const AnsiString AData)
{
   thread->Log("< "+AData);
}
//---------------------------------------------------------------------------

void __fastcall TQSLogEvent::LogEventDisconnect(TIdConnectionIntercept *ASender)
{
}

//---------------------------------------------------------------------------
// TQSHTTP
//---------------------------------------------------------------------------

__fastcall TQSHTTP::TQSHTTP(TComponent *Owner)
   : TIdHTTP(Owner)
{
   Content = new TMemoryStream();
   OnWorkBegin = HTTPEventWorkBegin;
   OnWorkEnd = HTTPEventWorkEnd;
   OnWork = HTTPEventWork;
}
//---------------------------------------------------------------------------

__fastcall TQSHTTP::~TQSHTTP()
{
   delete Content;
}
//---------------------------------------------------------------------------

void __fastcall TQSHTTP::HTTPEventWorkBegin(TObject *ASender, TWorkMode AWorkMode,
   __int64 AWorkCountMax)
{
   PostMessage(((TForm*)Owner)->Handle, WM_PROGRESS, 0, (LPARAM)AWorkCountMax);
}
//---------------------------------------------------------------------------

void __fastcall TQSHTTP::HTTPEventWork(TObject *ASender, TWorkMode AWorkMode,
   __int64 AWorkCount)
{
   PostMessage(((TForm*)Owner)->Handle, WM_PROGRESS, 1, (LPARAM)AWorkCount);
   if (*cancel) {
	  Abort();
   }
}
//---------------------------------------------------------------------------

void __fastcall TQSHTTP::HTTPEventWorkEnd(TObject *ASender, TWorkMode AWorkMode)
{
   PostMessage(((TForm*)Owner)->Handle, WM_PROGRESS, 2, (LPARAM)0);
}
//---------------------------------------------------------------------------

