/*
*  mimedecs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "MIMEdecs.h"
#include "utility.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
TMIMEdecoder *MIMEdecoder;
const int bufsize = 1024;
char buffer[bufsize];
//---------------------------------------------------------------------------

bool __fastcall DecodeMIMEMessage(TForm *Owner, TStrings *Message,
   const AnsiString &DisposeDir, TStrings *filenames)
{
   bool ok1, ok2;
   DecodeMIMEHeaders(Message);
   MIMEdecoder = new TMIMEdecoder(Owner, Message, DisposeDir);
   ok1 = MIMEdecoder->DecodeMessage();
   ok2 = MIMEdecoder->DecodeBody();
   filenames->Text = MIMEdecoder->filenames->Text;
   delete MIMEdecoder;
   return ok1 && ok2;
}

//---------------------------------------------------------------------------

bool __fastcall DecodeMIMEHeaders(TStrings *msg)
{
   bool dec = false;
   for (int i = 0; i < msg->Count; i++)
   {
	  AnsiString decoded, hdr = msg->Strings[i].Trim();
	  if (hdr.IsEmpty())
		 break;

	  if (hdr.LowerCase().Pos("=?iso-8859-1?q?"))
	  {
		 decoded = DecodeMIMEHeader(hdr);
		 if (decoded != hdr)
		 {
			msg->Strings[i] = decoded;
			dec = true;
		 }
	  }
   }
   return dec;
}
//---------------------------------------------------------------------------

AnsiString __fastcall DecodeMIMEHeader(AnsiString header)
{
   int p, len;
   AnsiString decoded, word, hex = "0123456789abcdefABCDEF";

   bool ok = true, dec = false;
   while (ok && (p = header.LowerCase().Pos("=?iso-8859-1?q?")) > 0)
   {
	  ok = false;
	  for (int i = p + 1; i <= header.Length(); i++)
	  {
		 if (header[i] == '=')
		 {
			if (i == header.Length() || hex.Pos(header[i + 1]) == 0)
			{
			   len = i - (p - 1);
			   word = header.SubString(p, len);
			   decoded = decodeqencoding(word);
			   header.Delete(p, len);
			   header.Insert(decoded, p);
			   dec = true;
			   ok = true;
			   break;
			}
		 }
	  }
   }

   return dec ? header.c_str() : "";
}
//---------------------------------------------------------------------------

AnsiString __fastcall decodeqencoding(AnsiString &word)
{
   AnsiString decoded, text, code;
   word = word.SubString(16, 1024);

   int p, x = 0;
   while ((text = GetTok(x, word, '?')) != "")
   {
	  if (text == "=")
		 break;

	  while ((p = text.Pos("_")) > 0)
		 text[p] = ' ';

	  while ((p = text.Pos("=")) > 0)
	  {
		 code = "0x" + text.SubString(p + 1, 2);
		 char c = (char)code.ToIntDef(0);
		 if (c > 0)
		 {
			text.Delete(p, 3);
			text.Insert(c, p);
		 }
	  }

	  decoded += text;
	  x++;
   }
   return decoded;
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall DecodeMIMEHeader(AnsiString header)
{
   if (header.LowerCase().Pos("=?iso-8859-1?q?") == 0)
	  return "";

   AnsiString text, decoded;
   int x = 0;
   while ((text = GetTok(x, header, ' ')) != "")
   {
	  if (text[1] == '"')
		text = dropquotes(text);

	  decoded += " ";
	  if (text.LowerCase().Pos("=?iso-8859-1?q?") == 1)
	  {
		 text = decodeqencoding(text);
	  }
	  decoded += text;

	  x++;
   }

   return decoded.Trim();
}*/

//---------------------------------------------------------------------------
// TMIMEdecoder
//---------------------------------------------------------------------------

__fastcall TMIMEdecoder::TMIMEdecoder(TForm *owner, TStrings *Message,
   const AnsiString &DisposeDir)
{
   Owner = owner;
   disposedir = DisposeDir;
   if (disposedir.Length() && disposedir[disposedir.Length()] != '\\')
	  disposedir += "\\";
   parts = new TMIMEpartsList();
   filenames = new TStringList();
   msg = Message;
}
//---------------------------------------------------------------------------

__fastcall TMIMEdecoder::~TMIMEdecoder()
{
   delete filenames;
   delete parts;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::DecodeMessage()
{
   TMIMEpart *part;
   content = GetPrimeMIMEContent();
   AnsiString boundary = GetContent("boundary=").Trim();

   if (!boundary.IsEmpty())
   {
	  int fromline = eoh() + 1;

	  while ((part = GetNextPart(fromline, boundary)) != NULL)
		 parts->AddPart(part->contenttype, part);

	  while (fromline < msg->Count)
		 msg->Delete(fromline);

	  ConstructMessage();
   }

   return true;
}
//---------------------------------------------------------------------------

TMIMEpart* __fastcall TMIMEdecoder::GetNextPart(int &fromline, const AnsiString &Boundary)
{
   while (fromline < msg->Count)
   {
	  if (msg->Strings[fromline].Pos("--" + Boundary) == 1)
		 break;
	  msg->Delete(fromline);
   }

   if (fromline < msg->Count && msg->Strings[fromline] == "--" + Boundary + "--")
	  return NULL;

   msg->Delete(fromline);
   AnsiString line, boundary;
   TStringList *headers = new TStringList();
   while (fromline < msg->Count)
   {
	  line = msg->Strings[fromline];
	  msg->Delete(fromline);
	  headers->Add(line);

	  if (line.Trim().IsEmpty())
		 break;
   }

   GetMIMEContent(headers);
   boundary = GetContent("boundary=");

   TMIMEpart *part;
   if (!boundary.IsEmpty())
   {
	  while ((part = GetNextPart(fromline, boundary)) != NULL)
	  {
		 parts->AddPart(part->contenttype, part);
		 part->GetFields();
	  }

	  return GetNextPart(fromline, Boundary);
   }

   part = new TMIMEpart(disposedir);
   part->headers->AddStrings(headers);
   part->GetFields();
   delete headers;

   AnsiString text = " decoding " + part->Name();
 //Owner->Perform(WM_LOGATTACHMENT, 0, (LPARAM)text.c_str());

   AnsiString encoding = part->encoding;
   while (fromline < msg->Count)
   {
	  line = msg->Strings[fromline];
	  if (line.Pos(Boundary) > 0)
		 break;

	  if (encoding == "quoted-printable") DecodeQuotedPrintableLine(line, part); else
	  if (encoding == "x-uuencode") DecodeUUencodeLine(line, part); else
	  if (encoding == "base64") DecodeBase64Line(line, part); else
	  if (encoding == "7bit") Process7bitLine(line, part); else
		 ProcessUnknownLine(line, part);

	  msg->Delete(fromline);
   }

   part->data->Position = 0;
   return part;
}
//---------------------------------------------------------------------------

void __fastcall TMIMEdecoder::ConstructMessage()
{
   AnsiString type;
   bool hastext = false;
   TStringList *body = new TStringList();

   for (int i = 0; i < parts->Count; i++)
   {
	  TMIMEpart *part = parts->GetPart(i);
	  type = part->contenttype;

	  if (type.Pos("text/plain"))
	  {
		 body->LoadFromStream(part->data);
		 msg->AddStrings(body);
		 hastext = true;
		 parts->Delete(i);
		 delete part;
		 break;
	  }
   }

   if (!hastext)
   {
	  for (int i = 0; i < parts->Count; i++)
	  {
		 TMIMEpart *part = parts->GetPart(i);
		 type = part->contenttype;

		 if (type.Pos("text/html"))
		 {
			body->LoadFromStream(part->data);
			msg->AddStrings(body);
			hastext = true;
			parts->Delete(i);
			delete part;
			break;
		 }
	  }
   }

   if (!hastext)
   {
	  for (int i = 0; i < parts->Count; i++)
	  {
		 TMIMEpart *part = parts->GetPart(i);
		 type = part->contenttype;

		 if (type.Pos("text/") == 0)
		 {
			body->LoadFromStream(part->data);
			msg->AddStrings(body);
            parts->Delete(i);
            delete part;
            break;
         }
      }
   }

   for (int i = 0; i < parts->Count; i++)
   {
      TMIMEpart *part = parts->GetPart(i);
      if (part->Dispose())
         filenames->Add(part->GetCompatibleFilename());
      //parts->GetPart(i)->Dispose();
   }
   parts->Clear();

   delete body;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::DecodeBody()
{
   TMIMEpart *part;

   int fromline = eoh() + 1;
   while ((part = GetNextPartBody(fromline)) != NULL)
	  parts->AddPart(part->contenttype, part);
   TrimNulls(msg);

   // dispose the parts that were found
   for (int i = 0; i < parts->Count; i++)
   {
	  TMIMEpart *part = parts->GetPart(i);
	  if (part->Dispose())
		 filenames->Add(part->GetCompatibleFilename());
	  //parts->GetPart(i)->Dispose();
   }
   parts->Clear();


   return true;
}
//---------------------------------------------------------------------------

TMIMEpart* __fastcall TMIMEdecoder::GetNextPartBody(int &fromline)
{
   // find an attachment by content-type line
   while (fromline < msg->Count)
   {
      if (msg->Strings[fromline].Pos("Content-Type") == 1)
         break;
      fromline++;
   }

   // not there? quit
   if (fromline == msg->Count)
      return NULL;

   // get the attachment headers
   AnsiString line;
   int x = fromline;
   TStringList *headers = new TStringList();
   while (x < msg->Count)
   {
      line = msg->Strings[x];
      if (line.Trim().IsEmpty())
         break;

      headers->Add(line);
	  x++;
   }

   bool ok = true;
   // are they actually mime attachment headers?
   for (int i = 0; i < headers->Count - 1; i++)
   {
	  line = headers->Strings[i].LowerCase();
      if (line.Pos("content-") != 1 && line.Pos("\t") != 1 && line.Pos(" ") != 1)
      {
         ok = false;
         break;
      }
   }

   if (!ok) // no, not headers
   {
      delete headers;
      return NULL;
   }

   // delete headers from message
   while (fromline < msg->Count)
   {
      line = msg->Strings[fromline];
      msg->Delete(fromline);
      if (msg->Strings[fromline].Trim().IsEmpty())
         break;
   }

   // create part and add headers
   TMIMEpart *part = new TMIMEpart(disposedir);
   part->headers->AddStrings(headers);
   part->GetFields();
   delete headers;

   msg->Delete(fromline); // delete empty line
   AnsiString text = " decoding " + part->Name();
 //Owner->Perform(WM_LOGATTACHMENT, 0, (LPARAM)text.c_str());

   // move attachment to the part's memorystream
   AnsiString encoding = part->encoding;
   while (fromline < msg->Count)
   {
      line = msg->Strings[fromline];
      if (line.Trim().IsEmpty())
         break;

      if (encoding == "quoted-printable") DecodeQuotedPrintableLine(line, part); else
      if (encoding == "x-uuencode") DecodeUUencodeLine(line, part); else
      if (encoding == "base64") DecodeBase64Line(line, part); else
      if (encoding == "7bit") Process7bitLine(line, part); else
         ProcessUnknownLine(line, part);

      msg->Delete(fromline);
   }

   // okie dokie
   part->data->Position = 0;
   return part;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEdecoder::GetContent(const AnsiString &Field)
{
   AnsiString item;
   int i = 0; while((item = GetTok(i, content, ';').Trim()) != "")
   {
      if (item.Trim().LowerCase().Pos(Field) == 1)
         return dropquotes(item.SubString(Field.Length() + 1, item.Length()));
      i++;
   }
   return "";
}
//---------------------------------------------------------------------------

int __fastcall TMIMEdecoder::eoh()
{
   int i = 0;
   while (i < msg->Count)
   {
      if (msg->Strings[i].Trim().IsEmpty())
         break;

      i++;
   }
   return i;
}
//---------------------------------------------------------------------------

void __fastcall TMIMEdecoder::GetMIMEContent(TStrings *headers)
{
   int i;
   content = "";
   AnsiString line;
   for (i = 0; i < headers->Count; i++)
   {
      if (content.Length() && content[content.Length()] != ';')
         content += ';';
      content += headers->Strings[i];
   }

   while ((i = content.Pos(" ")) > 0)
      content.Delete(i, 1);
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::DecodeQuotedPrintableLine(AnsiString &line, TMIMEpart *part)
{
   strcpy(buffer, AnsiString(line + "\r\n").c_str());
   int size = DecodeQuotedPrintableLine(buffer);
   part->data->Write(buffer, size);
   return status == statMimeOk;
}
//---------------------------------------------------------------------------
#define SKIP '\202'

const char hexmap[] = {
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
      0 ,    1 ,    2 ,    3 ,    4 ,    5 ,    6 ,    7 ,
      8 ,    9 ,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,    10,    11,    12,    13,    14,    15,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,
     SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP,   SKIP
};

int  __fastcall TMIMEdecoder::DecodeQuotedPrintableLine(char *line)
{
  ok = true;
  status = statMimeOk;
  char *s = line;
  char *finalresult = (char*)calloc(strlen(line) + sizeof(char), sizeof(char));
  char *result = finalresult;
  while (*s != '\0') //loop through the entire string...
  {
    if (*s == '=') //woops, needs to be decoded...
    {
      for (int i = 0; i < 3; i++) //is s more than 3 chars long...
      {
        if (s[i] == '\0')
        {
          //error in the decoding...
          //MsgDlg(mimedlg,
          //   s("Encountered corruption while decoding 'quoted-printable'\n")+
          //   s("MIME part. Skipping..."),"MIME Error",MB_OK);
          //status = statMimeDecodeError;
          //return ok;
        }
      }
      char mid[3];
      s++; //move past the "="
      //let's put the hex part into mid...
      ok = true;
      for (int i = 0; i < 2; i++)
      {
        if (hexmap[s[i]] == SKIP)
        {
          //we have an error, or a linebreak, in the encoding...
          ok = false;
          if (s[i] == '\r' && s[i + 1] == '\n')
          {
            s += 2;
            //*(result++) = '\r';
            //*(result++) = '\n';
            break;
          }
          else
          {
            //we have an error in the encoding...
            status = 1;
            //s--;
          }
        }
        mid[i] = s[i];
      }
      //now we just have to convert the hex string to an char...
      if (ok)
      {
        s += 2;
        int m = hexmap[mid[0]];
        m <<= 4;
        m |= hexmap[mid[1]];
        *(result++) = (char)m;
      }
    }
    else
    {
      if (*s != '\0') *(result++) = *(s++);
    }
  }
  strcpy(line, finalresult);
  delete finalresult;
  return strlen(line);
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::DecodeBase64Line(AnsiString &line, TMIMEpart *part)
{
   strcpy(buffer, line.c_str());
   int size = DecodeBase64Line(buffer);
   part->data->WriteBuffer(buffer, size);
   return status == statMimeOk;
}
//---------------------------------------------------------------------------

int __fastcall TMIMEdecoder::DecodeBase64Line(char *line)
{
   ok = true;
   status = statMimeOk;
   byte ch,inbuf[3],outbuf[4];
   bool flignore,flendtext=false;
   unsigned ixtext=0,oxtext=0,lentext=strlen(line);
   short ixinbuf=0;

   while(ixtext<lentext)
   {
      ch = line[ixtext++];
      flignore = false;

      if ((ch >= 'A') && (ch <= 'Z')) ch = (byte)(ch - 'A'); else
      if ((ch >= 'a') && (ch <= 'z')) ch = (byte)(ch - 'a' + 26); else
      if ((ch >= '0') && (ch <= '9')) ch = (byte)(ch - '0' + 52); else
      if ( ch == '+') ch = 62; else
      if ( ch == '=') flendtext = true; else
      if ( ch == '/') ch = 63; else
         flignore = true;

      if (!flignore)
      {
         short charsinbuf = 3;
         boolean flbreak = false;
         if (flendtext)
         {
            if (ixinbuf==0) break;

            if ((ixinbuf==1) || (ixinbuf==2)) charsinbuf = 1;
            else charsinbuf = 2;

            flbreak = true;
            ixinbuf = 3;
         }

         inbuf[ixinbuf++] = ch;
         if (ixinbuf==4)
         {
            ixinbuf = 0;
            outbuf[0] = (byte)((inbuf[0] << 2) | ((inbuf[1] & 0x30) >> 4));
            outbuf[1] = (byte)(((inbuf[1] & 0x0F) << 4) | ((inbuf[2] & 0x3C) >> 2));
            outbuf[2] = (byte)(((inbuf[2] & 0x03) << 6) | ( inbuf[3] & 0x3F));
            for (int i=0; i<charsinbuf; i++)
               line[oxtext++] = outbuf[i];
         }
         if (flbreak) break;
      }
   }
   line[oxtext] = '\0';
   return oxtext;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::DecodeUUencodeLine(AnsiString &line, TMIMEpart *part)
{
   strcpy(buffer,line.c_str());
   int size=DecodeUUencodeLine(buffer);
   part->data->WriteBuffer(buffer,size);
   return status == statMimeOk;
}
//---------------------------------------------------------------------------
#define DEC(c)	(((c) - ' ') & 077) // single character decode

int  __fastcall TMIMEdecoder::DecodeUUencodeLine(char *line)
{
    ok = true;
    status = statMimeOk;
    if (AnsiString(line).Pos("begin ")==1 || AnsiString(line).Pos("end")==1)
       return 0;

    int n,c1,c2,c3;
    char buf[bufsize],*bp,ch;
    unsigned ixtext=0,oxtext=0,lentext=strlen(line);

    while(ixtext < lentext)
    {
        for (int i=0; i<bufsize; i++)
        {
           if (ixtext>=lentext) break;
           ch = line[ixtext++];
           if (ch=='\r') continue;
           if (ch=='\n') break;
           buf[i] = ch;
        }

	n = DEC(buf[0]); if (n<=0) break;
	bp = &buf[1];
	while (n > 0)
        {
           c1 = ((DEC(bp[0]) << 2) | (DEC(bp[1]) >> 4));
           c2 = ((DEC(bp[1]) << 4) | (DEC(bp[2]) >> 2));
           c3 = ((DEC(bp[2]) << 6) |  DEC(bp[3]));
           if (n>=1) line[oxtext++] = (char)c1;
           if (n>=2) line[oxtext++] = (char)c2;
           if (n>=3) line[oxtext++] = (char)c3;
		   bp += 4;
		   n -= 3;
		}
	}
    line[oxtext] = '\0';
    return oxtext;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::ProcessUnknownLine(AnsiString &line, TMIMEpart *part)
{
   if (part->data->Size == 0)
   {
      part->data->Write(part->headers->Text.c_str(), part->headers->Text.Length());
      part->data->Write("\r\n", 2);
   }
   part->data->Write(line.c_str(), line.Length());
   part->data->Write("\r\n", 2);
   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEdecoder::Process7bitLine(AnsiString &line, TMIMEpart *part)
{
   part->data->Write(line.c_str(), line.Length());
   part->data->Write("\r\n", 2);
   return true;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEdecoder::GetPrimeMIMEContent()
{
   AnsiString line, content;
   int i = -1; while(++i < msg->Count)
   {
      if ((line = msg->Strings[i].Trim()).IsEmpty())
         break;

      if (line.Pos("content-") == 0 && line.Pos("boundary=") == 0)
         continue;

      if (content.Length() && content[content.Length()] != ';')
         content += ';';
      content += line;

      while(++i < msg->Count)
      {
         if ((line = msg->Strings[i].Trim()).IsEmpty())
            break;

         if (AnsiString(" \t").Pos(line[1]) > 0)
         {
            if (content.Length() && content[content.Length()] != ';')
               content += ';';
            content += " " + line.Trim();
            continue;
         }
         i--;
         break;
      }
      if (line.IsEmpty())
         break;
   }
   content = content.Trim();
   return content;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEdecoder::GetMIMEVersion()
{
   AnsiString line,version;
   for (int i = 0; i < msg->Count; i++)
   {
      line = msg->Strings[i].LowerCase();
      if (line.Trim().IsEmpty())
         break;

      if (line.Pos("mime-version: ")==1)
      {
         version = line.SubString(14,16);
         break;
      }
   }
   return version;
}
//---------------------------------------------------------------------------
// TMIMEpart
//---------------------------------------------------------------------------

__fastcall TMIMEpart::TMIMEpart(const AnsiString &DisposeDir)
   : TObject()
{
   data = new TMemoryStream();
   headers = new TStringList();
   //buffer = new char[bufsize];
   disposedir = DisposeDir;
}
//---------------------------------------------------------------------------

__fastcall TMIMEpart::~TMIMEpart()
{
   //delete [] buffer;
   delete headers;
   delete data;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEpart::GetCompatibleFilename()
{
   assert(!filename.IsEmpty());
 //AnsiString badchars = "\ / : * ? \" < > |";
   AnsiString goodchars =
      "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
      "1234567890=_.#$%^&()-+~;@!,";

   AnsiString okfilename = filename;
   for (int i = 1; i <= okfilename.Length(); i++)
   {
      if (goodchars.Pos(okfilename[i]) == 0)
         okfilename[i] = '_';
   }

   return okfilename;
}
//---------------------------------------------------------------------------

void __fastcall TMIMEpart::GetFields()
{
   content = ""; // get all mime fields into 1 line
   if (!headers->Count)
      return;
   assert(headers->Count);
   int p, i = 0;
   while (i < headers->Count && !headers->Strings[i].Trim().IsEmpty())
   {
      if (content.Length() && content[content.Length()] != ';')
         content += ';';
      content += headers->Strings[i].Trim().LowerCase();
      i++;
   }
   while ((p = content.Pos(" ")) > 0) // remove blanks
      content.Delete(p, 1);

   // fill the part variables
   filename = dropquotes(GetContent("filename="));
   contname = dropquotes(GetContent("name="));
   encoding = GetContent("content-transfer-encoding:");
   disposition = GetContent("content-disposition:");
   contenttype = GetContent("content-type:");
}
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEpart::GetContent(const AnsiString &field)
{
   AnsiString header, tok, line;

   int p, i = 0;
   while ((tok = GetTok(i++, content, ';')) != "" && (p = tok.Pos(field)) != 1)
      continue;

   if (p == 1)
   {
      return tok.SubString(field.Length() + 1, 64);
   }

   return "";
}
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEpart::Name()
{
   AnsiString name = filename;
   if (name.IsEmpty())
   {
      name = contname;
      if (name.IsEmpty())
      {
         name = contenttype;
         if (name.IsEmpty())
            name = "part (" + IntToStr(data->Size) + ") ";
      }
   }
   return name;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEpart::Dispose()
{
   if (!filename.IsEmpty())
   {
      data->SaveToFile(disposedir + GetCompatibleFilename());
      return true;
   }
   return false;
}

//---------------------------------------------------------------------------
// TMIMEpartsList
//---------------------------------------------------------------------------

__fastcall TMIMEpartsList::TMIMEpartsList()
   : TStringList()
{
}
//---------------------------------------------------------------------------

__fastcall TMIMEpartsList::~TMIMEpartsList()
{
   Clear();
}
//---------------------------------------------------------------------------

TMIMEpart* __fastcall TMIMEpartsList::GetPart(int partno)
{
   assert(partno >= 0 && partno < Count);
   return (TMIMEpart*)Objects[partno];
}
//---------------------------------------------------------------------------

int __fastcall TMIMEpartsList::AddPart(const AnsiString &name, TMIMEpart *part)
{
   return AddObject(name, part);
}
//---------------------------------------------------------------------------

void __fastcall TMIMEpartsList::Clear()
{
   for (int i = 0; i < Count; i++)
      delete Objects[i];
   TStringList::Clear();
}

//---------------------------------------------------------------------------
