/*
*  mimeencs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
#include <sys\stat.h>
#include <stdio.h>
#include <math.h>
#include <registry.hpp>

#include "mimeencs.h"
#include "utility.h"
#include "global.h"
#include "qsdbs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
TMIMEencoder *MIMEencoder;
//---------------------------------------------------------------------------

bool __fastcall HasAttachments(TStrings *message)
{
   bool has=false;
   for (int i=0; i<message->Count; i++)
   {
	  AnsiString line = message->Strings[i].Trim().LowerCase();
      if (line.IsEmpty()) break;
      if (line.Pos("attachment: ")==1)
      {
         has = true;
         break;
      }
   }
   return has;
}
//---------------------------------------------------------------------------

bool __fastcall EncodeMIMEAttachments(TForm *Owner, TStrings *Message, bool Inline)
{
   if (!HasAttachments(Message))
   {
	  MsgBox(Owner,
		 "No attachments. Nothing to encode!",
		  MT_ERROR, MB_OK);
	  return true;
   }

   MIMEencoder = new TMIMEencoder(Owner, Message, Inline);
   bool ok = MIMEencoder->EncodeAttachments();
   delete MIMEencoder;
   return ok;
}

//---------------------------------------------------------------------------
// TMIMEencoder
//---------------------------------------------------------------------------

__fastcall TMIMEencoder::TMIMEencoder(TForm *owner, TStrings *message, bool InLine)
{
   Owner = owner;
   status = statMimeOk;
   Inlines = new TStringList();
   Filenames = new TStringList();
   Message = message;
   Inline = InLine;
   ok = true;
}
//---------------------------------------------------------------------------

__fastcall TMIMEencoder::~TMIMEencoder()
{
   delete Filenames;
   delete Inlines;
}
//---------------------------------------------------------------------------

void __fastcall TMIMEencoder::DeleteHeader(const AnsiString &Header)
{
   for (int i = 0; i< Message->Count; i++)
   {
      AnsiString hdr, line = Message->Strings[i].Trim();
      if (line.Trim().IsEmpty())
         break;
      if (Message->Strings[i].LowerCase().Pos(Header.LowerCase()) == 1)
      {
         Message->Delete(i);
         while (i < Message->Count)
         {
            hdr = Message->Strings[i].TrimRight();
			if (hdr.IsEmpty())
			   break;

            if (hdr[1] == ' ' || hdr[1] == '\t')
               Message->Delete(i);
			else
			   break;
         }
         break;
      }
   }
}
//---------------------------------------------------------------------------

int __fastcall TMIMEencoder::EOH()
{
   int eoh = Message->Count-1;
   for (int i=0; i<Message->Count; i++)
      if (Message->Strings[i].Trim().IsEmpty())
      {
         eoh = i;
         break;
      }
   if (eoh==Message->Count-1)
      Message->Add("");
   return eoh;
}
//---------------------------------------------------------------------------

TStream* __fastcall TMIMEencoder::new_AttachmentStream(const AnsiString &Filename)
{
#ifdef QS2
   if (Filename.Pos("DB:\\") == 1)
   {
	  TMemoryStream *f = new TMemoryStream();
	  DB->LoadAttachment(GetTok(1, Filename, '\\'), GetTok(2, Filename, '\\'), f);
	  f->Position = 0;
	  return f;
   }
#endif
   return new TFileStream(Filename, fmOpenRead);
}
//---------------------------------------------------------------------------

AnsiString __fastcall datekey(const AnsiString &AttachmentShortKey)
{
   return GetTok(1, AttachmentShortKey, '\\');
}
//---------------------------------------------------------------------------

AnsiString __fastcall partno(const AnsiString &AttachmentShortKey)
{
   return GetTok(2, AttachmentShortKey, '\\');
}
//---------------------------------------------------------------------------

int __fastcall TMIMEencoder::GetAttachmentNames()
{
   AnsiString attname,attline,line,item,filename;
   int i = -1; while(++i < Message->Count)
   {
	  line = Message->Strings[i];
	  if (line.Trim().IsEmpty())
		 break;

	  if (line.LowerCase().Pos("attachment: ")==1)
	  {
		 Message->Delete(i);
		 attline = line.Trim().SubString(13,1024);
		 if (attline.SubString(attline.Length(),1)!=";") attline += ";";
		 while(i < Message->Count)
		 {
			line = Message->Strings[i];
			if (AnsiString(" \t").Pos(line.SubString(1, 1)) > 0)
			{
			   attline += line.Trim();
			   if (attline.SubString(attline.Length(),1) != ";")
				  attline += ";";
			   Message->Delete(i);
			   continue;
			}
			break;
		 }
	  }
   }
   if (attline.IsEmpty())
	  return 0;

   int pos,count=0;
   while ((pos=attline.Pos("\r"))>0) attline.Delete(pos,1);
   while ((pos=attline.Pos("\n"))>0) attline.Delete(pos,1);
   while ((pos=attline.Pos("\t"))>0) attline.Delete(pos,1);
   while ((pos=attline.Pos("\""))>0) attline.Delete(pos,1);
   while ((pos=attline.Pos(" ;"))>0) attline.Delete(pos,1);
   while ((pos=attline.Pos("; "))>0) attline.Delete(pos+1,1);
   while ((attname=GetTok(count++,attline,';').Trim())!="")
   {
	  filename = GetTok(0,attname,',');
#ifdef QS2
	  if (!FileExists(filename) && !DB->AttachmentExists(datekey(filename), partno(filename)))
	  {
		 if (MsgBox(Owner,
			"The file attachment \""+filename+"\" doesn't exist!\n\n"
			"Continue without it?", MT_ERROR, MB_YESNO) == IDNO)
		 {
			status = statMimeBadFilename;
			return 0;
		 }
	  }
#else
	  if (!FileExists(filename))
	  {
		 if (MsgBox(Owner,
			"The file attachment \""+filename+"\" doesn't exist!\n\n"
			"Continue without it?", MT_ERROR, MB_YESNO) == IDNO)
		 {
			status = statMimeBadFilename;
			return 0;
		 }
	  }
#endif
	  //bool Inline = false;
	  //int i=0; while((item = GetTok(++i, attname, ',')) != "")
	  //{
	  //   if (item.Trim().LowerCase() == "body")
	  //   {
	  //      Inline = true;
	  //      break;
      //   }
	  //}
	  if (!Inline)
         Filenames->Add(attname);
      else
         Inlines->Add(attname);
   }

   DeleteHeader("Attachment: ");
   return count;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEencoder::AttachFilesInline(TStrings *filenames)
{
   ok = true;
   status = statMimeOk;
   TStrings *msg = Message;
   TStringList *tail = new TStringList();

   msg->BeginUpdate();
   TrimNullsAfter(msg);
   int idx = msg->IndexOf("~~");
   if (idx>0) while(idx<msg->Count)
   {
      tail->Add(msg->Strings[idx]);
      msg->Delete(idx);
   }
   TrimNullsAfter(msg); msg->Add("");
   for (int i=0; i<filenames->Count; i++)
   {
      AnsiString method, fname, FileName, filename = filenames->Strings[i];
      FileName = GetTok(0, filename, ',');
      method = GetTok(1, filename, ',');

      fname = ExtractFileName(FileName);
	  Owner->Perform(WM_LOGATTACHMENT, 0, (LPARAM)fname.c_str());
	  Owner->Perform(WM_SHOWSTATUS, 0,
         (LPARAM)AnsiString(" encoding " + fname).c_str());

      if (method=="x-uuencode")
         ok = AttachInlineUU(FileName, msg);
      else
		 ok = AttachInlineBase64(FileName,msg);

      if (!ok)
      {
         msg->EndUpdate();
         delete msg;
      }
   }

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)" ok ");

   msg->AddStrings(tail);
   msg->EndUpdate();

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEencoder::AttachInlineBase64(const AnsiString &filename, TStrings* workmsg)
{
   ok = true;
   status = statMimeOk;
   AnsiString justname = ExtractFileName(filename);

   workmsg->BeginUpdate();
   workmsg->Add("Content-Type: "+GetContentType(justname)+"; name=\""+justname+"\"");
   workmsg->Add("Content-Disposition: attachment; filename=\""+justname+"\"");
   workmsg->Add("Content-Transfer-Encoding: base64");
   workmsg->Add("");
   Base64Encode(filename, workmsg);
   workmsg->Add("");
   workmsg->EndUpdate();

   return ok;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEencoder::AttachInlineUU(AnsiString filename, TStrings* workmsg)
{
   ok = true;
   status = statMimeOk;
   AnsiString justname = ExtractFileName(filename);

   workmsg->BeginUpdate();
   workmsg->Add("Content-Type: "+GetContentType(justname)+"; name=\""+justname+"\"");
   workmsg->Add("Content-Disposition: attachment; filename=\""+justname+"\"");
   workmsg->Add("Content-Transfer-Encoding: x-uuencode");
   workmsg->Add("");
   UUEncode(filename, workmsg);
   workmsg->Add("");
   workmsg->EndUpdate();

   return ok;
}
//---------------------------------------------------------------------------

AnsiString __fastcall GetBoundaryNumberEx()
{
   AnsiString number,boundary;
   long double num = (double)Now()*(double)GetTickCount();
   boundary = number.FloatToStrF(num,AnsiString::sffFixed,15,4);
   int pos=boundary.Pos(".");
   if (pos>0) boundary[pos] = '_';
   return boundary;
}
//---------------------------------------------------------------------------

bool __fastcall TMIMEencoder::EncodeAttachments()
{
   ok = true;
   status = statMimeOk;
   TStrings *msg = Message;
   int cutidx;

   if (!GetAttachmentNames() || status==statMimeBadFilename)
	  return false;
//look(msg,"encode");
   if ((cutidx = msg->IndexOf("~~")) > 0)
	  while(msg->Count > cutidx)
		 msg->Delete(cutidx);

   if (Inlines->Count)
	  ok = AttachFilesInline(Inlines);

   if (!ok)
	  return ok;

   if (!Filenames->Count)
	  return ok;

   MimeBoundary = "==0000_MIME_Boundary_"+GetBoundaryNumberEx()+"==";
   DeleteHeader("MIME-Version: ");
   DeleteHeader("Content-Type: ");
   AnsiString line;
   int idx = -1; while(++idx < msg->Count)
   {
	  line = msg->Strings[idx];
	  if (line.IsEmpty())
		 break;

	  if (line.LowerCase().Pos("from: ") == 1)
		 break;
   }
   msg->Insert(  idx, "MIME-Version: 1.0");
   msg->Insert(++idx, "Content-Type: multipart/mixed;");
   msg->Insert(++idx, "        boundary=\""+MimeBoundary+"\"");
   while(++idx < msg->Count && !msg->Strings[idx].Trim().IsEmpty())
	  continue;

   msg->Insert(++idx, "This is a multipart MIME message.");
   msg->Insert(++idx, "");
   msg->Insert(++idx, "--" + MimeBoundary);
   msg->Insert(++idx, "Content-Type: text/plain; charset=\"iso-8859-1\"");
   msg->Insert(++idx, "Content-Transfer-Encoding: 7bit");
   msg->Insert(++idx, "");

   AnsiString filename, encodemethod;
   int i = -1; while(++i < Filenames->Count)
   {
	  line = GetTok(0, Filenames->Strings[i], ';');
	  filename = GetTok(0, line, ',');
	  encodemethod = GetTok(1, line, ',').LowerCase();
	  if (encodemethod.IsEmpty()) encodemethod = "base64";
	  if (encodemethod=="uuencode") encodemethod = "x-uuencode";
	  if (AnsiString(",base64,x-uuencode,").Pos(","+encodemethod+",") == 0)
	  {
		 MsgBox(Owner,
			"The encode method, '"+encodemethod+"', specified is unknown to me\n"
			"It must be either 'base64' or 'x-uuencode'. Canceling...",
			 MT_ERROR, MB_OK);
		 status = statMimeBadEncodeMethod;
		 return (ok=false);
	  }
#ifdef QS2
	  if (!FileExists(filename) && !DB->AttachmentExists(datekey(filename), partno(filename)))
		 continue;
#else
	  if (!FileExists(filename))
		 continue;
#endif
	  AttachFile(filename, encodemethod);
   }

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)" ok ");
   if (status==statMimeOk)
   {
	  msg->Add("");
	  msg->Add("--"+MimeBoundary+"--");
	  //if (cutidx > 0)
	  //	 msg->Add("~~");
   }
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TMIMEencoder::AttachFile(const AnsiString &filename, const AnsiString &encodemethod)
{
   ok = true;
   status = statMimeOk;
   TrimNullsAfter(Message);
   Message->Add("");

   AnsiString justname = ExtractFileName(filename);
   Owner->Perform(WM_LOGATTACHMENT, 0, (LPARAM)justname.c_str());
   Owner->Perform(WM_SHOWSTATUS, 0,
      (LPARAM)AnsiString(" encoding " + justname).c_str());

   Message->Add("--" + MimeBoundary);
   Message->Add("Content-Type: "+GetContentType(justname)+"; name=\""+justname+"\"");
   Message->Add("Content-Disposition: attachment; filename=\""+justname+"\"");
   Message->Add("Content-Transfer-Encoding: "+encodemethod);
   Message->Add("");

   if (encodemethod == "base64")
      Base64Encode(filename, Message);
   else
      UUEncode(filename, Message);

   Snooze(10);
}
//---------------------------------------------------------------------------
// ENC is the basic 1 character encoding function to make a char printing
#define ENC(c) (((c) & 077) + ' ')

void __fastcall TMIMEencoder::UUEncode(AnsiString filename, TStrings *msg)
{
   ok = true;
   status = statMimeOk;

   filename = GetTok(0, filename, ',');
   AnsiString justname = ExtractFileName(filename);

   int mode = 666;
   msg->Add("begin "+IntToStr(mode)+" "+justname);

   int i,n,l;
   char buf[80];
   AnsiString buffer;

   TStream *f = new_AttachmentStream(filename);

   while(true)
   {
      n = f->Read(buf,45);
      buffer += (char)ENC(n);

      for (i=0; i<n; i += 3)
      {
         char *p = &buf[i];
	     buffer += (char)ENC(*p >> 2);
	     buffer += (char)ENC(((*p << 4) & 060) | ((p[1] >> 4) & 017));
	     buffer += (char)ENC(((p[1] << 2) & 074) | ((p[2] >> 6) & 03));
 	     buffer += (char)ENC(p[2] & 077);
	  }

      l = buffer.Length();
      for (i=1; i<=l; i++)
         if (buffer[i]==' ') buffer[i] = '`';
      msg->Add(buffer);
      buffer = "";

      if (n <= 0)
         break;
   }
   msg->Add("end");
   delete f;
}
//---------------------------------------------------------------------------

void __fastcall TMIMEencoder::Base64Encode(const AnsiString &Filename, TStrings *message)
{
   const char Base64Chars[] =
   {
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='
   };

   ok = true;
   status = statMimeOk;

   char DataIn[3];
   char DataOut[81];
   int i, Count = 0, BytesRd, InBytes = sizeof(DataIn);

   TStream *f = new_AttachmentStream(Filename);
   if (!f)
   {
	  status = statMimeFileOpenError;
	  ok = false;
	  return;
   }

   int c=-1;
   while (true)
   {
	  BytesRd=f->Read(DataIn,InBytes);
	  for (i=1; i<4; i++) DataOut[Count+i]=0x40;

	  if (fmod(++c, 20) == 0)
		 Application->ProcessMessages();

	  DataOut[Count]   = char((DataIn[0] & 0xFC) >> 2);
	  DataOut[Count+1] = char((DataIn[0] & 0x03) << 4);
	  if (BytesRd>1)
	  {
		 DataOut[Count+1] += char((DataIn[1] & 0xF0) >> 4);
		 DataOut[Count+2]  = char((DataIn[1] & 0x0F) << 2);
		 if (BytesRd>2)
		 {
			DataOut[Count+2] += char((DataIn[2] & 0xC0) >> 6);
			DataOut[Count+3]  =  char(DataIn[2] & 0x3F);
		 }
	  }

	  for (i=0; i<4; i++)
		 DataOut[Count+i] = byte(Base64Chars[DataOut[Count+i]]);
	  Count += 4;
	  if (Count<60 && BytesRd==InBytes)
		 continue;

	  DataOut[Count] = '\0';
	  message->Add(AnsiString(&DataOut[0]));

	  if (BytesRd<InBytes)
		 break;
	  Count=0;
   }
   delete f;
}
//---------------------------------------------------------------------------
// this didn't work in vista
/*AnsiString __fastcall TMIMEencoder::GetContentType(const AnsiString &Filename)
{
   AnsiString ext, mimeType = "application/octet-stream";
   ext = ExtractFileExt(Filename).LowerCase();
   TRegistry *reg = new TRegistry();
   try
   {
	  reg->RootKey = HKEY_CLASSES_ROOT;
	  if (reg->OpenKey(ext, false))
	  {
		 if (reg->ValueExists("Content Type"))
			mimeType = reg->ReadString("Content Type");
		 reg->CloseKey();
	  }
   }
   __finally
   {
	  delete reg;
   }
sm(mimeType);
   return mimeType;
}*/
//---------------------------------------------------------------------------

AnsiString __fastcall TMIMEencoder::GetContentType(const AnsiString &Filename)
{
  AnsiString ext = ExtractFileExt(Filename).LowerCase();
  if (AnsiString(".htm.html").Pos(ext))                return "text/html";
  if (AnsiString(".txt.log.asc.ini.cmd.rtf").Pos(ext)) return "text/plain";
  if (AnsiString(".au.snd").Pos(ext))                  return "audio/basic";
  if (AnsiString(".aif").Pos(ext))                     return "audio/aiff";
  if (AnsiString(".gif").Pos(ext))                     return "image/gif";
  if (AnsiString(".jpg.jpeg").Pos(ext))                return "image/jpeg";
  if (AnsiString(".png").Pos(ext))                     return "image/png";
  if (AnsiString(".avi").Pos(ext))                     return "video/avi";
  if (AnsiString(".rtf").Pos(ext))                     return "text/rtf";
  if (Filename.Pos("message/"))                        return "text/plain";
  return "application/octet-stream";
}

//---------------------------------------------------------------------------
// Utility
//---------------------------------------------------------------------------

/*AnsiString __fastcall Base64EncodeLine(AnsiString line)
{
   const char Base64Chars[] =
   {
	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
	'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
	'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm',
	'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/', '='
   };

   char DataIn[3];
   char DataOut[81];
   AnsiString encodedline;
   int i,Count=0,BytesRd,InBytes=sizeof(DataIn);

   TMemoryStream *f = new TMemoryStream();
   f->Write(line.c_str(), line.Length());
   f->Position = 0;

   while (true)
   {
      BytesRd=f->Read(DataIn,InBytes);
      for (i=1; i<4; i++) DataOut[Count+i]=0x40;

      DataOut[Count]   = char((DataIn[0] & 0xFC) >> 2);
      DataOut[Count+1] = char((DataIn[0] & 0x03) << 4);
      if (BytesRd>1)
      {
         DataOut[Count+1] += char((DataIn[1] & 0xF0) >> 4);
         DataOut[Count+2]  = char((DataIn[1] & 0x0F) << 2);
         if (BytesRd>2)
         {
            DataOut[Count+2] += char((DataIn[2] & 0xC0) >> 6);
            DataOut[Count+3]  =  char(DataIn[2] & 0x3F);
         }
      }

      for (i=0; i<4; i++) DataOut[Count+i] = byte(Base64Chars[DataOut[Count+i]]);
      Count += 4; if (Count<60 && BytesRd==InBytes) continue;

      DataOut[Count] = '\0';

      encodedline += AnsiString(&DataOut[0]);

      if (BytesRd<InBytes) break;
      Count=0;
   }
   delete f;
   return encodedline;
}*/
//---------------------------------------------------------------------------
