//---------------------------------------------------------------------------
#pragma hdrstop

#include "mixconsole.h"
#include "mixmaster.h"
#include "utility.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
const int bufsize = 32768;
//---------------------------------------------------------------------------

bool __fastcall MixExeMixMessage(TForm *Owner, TStrings *Message, AnsiString &Chain, int copies,
   bool ispost, TStrings *feedback)
{
   assert(Owner);
   assert(Message && Message->Count);
   assert(copies > 0 && copies < 21);
   assert(feedback);

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Mixmaster encrypting...");
   AnsiString cmdline = getcommandline(MIXexemixmessage, MIXExeMixMessageCmd);
   setfield("%msgtype%", ispost ? "--post" : "--mail", cmdline);

   TMemoryStream *input = new TMemoryStream();
   TMemoryStream *output = new TMemoryStream();
   Message->SaveToStream(input);
   feedback->Clear();

   TMIXConsole *con = new TMIXConsole(Owner);
   bool ok = con->Run(cmdline, NULL, input, output);
   if (ok)
   {
	  feedback->LoadFromStream(output);
   }
   else
   {
	  Snooze(300);
	  MsgBox(Owner, con->error->Text, MT_ERROR, MB_OK);
   }
   delete con;
   delete output;
   delete input;

   Owner->Perform(WM_SHOWSTATUS, 0, (LPARAM)"Done!");
   Snooze(200);

   return ok;
}
/*int __fastcall MixExeMixMessage(TStrings *Message, AnsiString &Chain, int copies,
   bool ispost, TStrings *Feedback)
{
   int ret;
   AnsiString line;
   BUFFER *msg, *fbk;

   msg = buf_new();
   assert(msg != NULL);
   msg->sensitive = true;

   fbk = buf_new();
   assert(fbk != NULL);
   fbk->sensitive = true;

   for (int i = 0; i < Message->Count; i++)
   {
	  line = Message->Strings[i] + "\n";
	  buf_append(msg, (byte*)line.c_str(), line.Length());
   }

   int msgtype = ispost ? MSG_POST : MSG_MAIL;
   if ((ret = mix_encrypt(msgtype, msg, Chain.c_str(), copies, fbk)) == 0)
	  mix_send();

   Feedback->Text = (char*)fbk->data;

   buf_free(msg);
   buf_free(fbk);

   return ret;
}*/

//---------------------------------------------------------------------------
// TMIXConsole
//---------------------------------------------------------------------------

__fastcall TMIXConsole::TMIXConsole(TForm *owner)
   : TObject()
{
   Owner = owner;
   error = new TStringList();
   attrib = new TStringList();
   status = new TStringList();
   statusstream = new TMemoryStream();
   errorstream = new TMemoryStream();
   inputevent = new TEvent(NULL, false, false, "", false);
   log = new TStringList();
}
//---------------------------------------------------------------------------

__fastcall TMIXConsole::~TMIXConsole()
{
   delete log;
   delete inputevent;
   delete errorstream;
   delete statusstream;
   delete status;
   delete attrib;
   delete error;
   //DestroyHandles();
}
//---------------------------------------------------------------------------

static void __fastcall CLOSEHandle(HANDLE &handle)
{
   if (handle)
   {
	  CloseHandle(handle);
	  handle = NULL;
   }
}
//---------------------------------------------------------------------------

bool __fastcall TMIXConsole::CreateHandles(const AnsiString &cmdLine)
{
   TSecurityAttributes sa;
   HANDLE hOutputReadTmp, hErrorReadTmp, hInputWriteTmp, hPassWriteTmp,
	  hAttrReadTmp, hStatReadTmp;

   bool bstatus = cmdLine.Pos("%stat%");
   bool battrib = cmdLine.Pos("%attr%");
   bool bpasswd = cmdLine.Pos("%pass%");

   DestroyHandles();
   sa.nLength = sizeof(sa);
   sa.lpSecurityDescriptor = NULL;
   sa.bInheritHandle = true;

   if (!CreatePipe(&hInputRead, &hInputWriteTmp, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!CreatePipe(&hOutputReadTmp, &hOutputWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!CreatePipe(&hErrorReadTmp, &hErrorWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (bstatus && !CreatePipe(&hStatReadTmp, &hStatWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (battrib && !CreatePipe(&hAttrReadTmp, &hAttrWrite, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (bpasswd && !CreatePipe(&hPassRead, &hPassWriteTmp, &sa, 0))
   {
	  ShowError("Pipe create error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hInputWriteTmp,
	  GetCurrentProcess(), &hInputWrite, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hOutputReadTmp,
	  GetCurrentProcess(), &hOutputRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (!DuplicateHandle(GetCurrentProcess(), hErrorReadTmp,
	  GetCurrentProcess(), &hErrorRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (bstatus && !DuplicateHandle(GetCurrentProcess(), hStatReadTmp,
	  GetCurrentProcess(), &hStatRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (battrib && !DuplicateHandle(GetCurrentProcess(), hAttrReadTmp,
	  GetCurrentProcess(), &hAttrRead, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (bpasswd && !DuplicateHandle(GetCurrentProcess(), hPassWriteTmp,
	  GetCurrentProcess(), &hPassWrite, 0, FALSE, DUPLICATE_SAME_ACCESS))
   {
	  ShowError("Duplicate handle error: \n");
	  return false;
   }

   if (battrib) CLOSEHandle(hAttrReadTmp);
   if (bstatus) CLOSEHandle(hStatReadTmp);
   if (bpasswd) CLOSEHandle(hPassWriteTmp);
   CLOSEHandle(hOutputReadTmp);
   CLOSEHandle(hErrorReadTmp);
   CLOSEHandle(hInputWriteTmp);

   return true;
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::DestroyHandles()
{
   CLOSEHandle(hInputRead);
   CLOSEHandle(hOutputRead);
   CLOSEHandle(hInputWrite);
   CLOSEHandle(hOutputWrite);
   CLOSEHandle(hErrorRead);
   CLOSEHandle(hErrorWrite);
   CLOSEHandle(hPassRead);
   CLOSEHandle(hPassWrite);
   CLOSEHandle(hAttrRead);
   CLOSEHandle(hAttrWrite);
   CLOSEHandle(hStatRead);
   CLOSEHandle(hStatWrite);
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::SetupFileDescriptors(AnsiString &cmdline)
{
   int p;
   AnsiString fd;

   p = cmdline.Pos("%stat%");
   if (p)
   {
	  fd = AnsiString((int)hStatWrite);
	  setfield("%stat%", fd, cmdline);
   }

   p = cmdline.Pos("%attr%");
   if (p)
   {
	  fd = AnsiString((int)hAttrWrite);
	  setfield("%attr%", fd, cmdline);
   }

   p = cmdline.Pos("%pass%");
   if (p)
   {
	  fd = AnsiString((int)hPassRead);
	  setfield("%pass%", fd, cmdline);
   }
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::PrepareStartupInfo()
{
   ZeroMemory(&startupinfo, sizeof(STARTUPINFO));
   startupinfo.cb = sizeof(STARTUPINFO);
   startupinfo.lpReserved = NULL;
   startupinfo.lpDesktop = NULL;
   startupinfo.lpTitle = NULL;
   startupinfo.lpReserved2 = NULL;
   startupinfo.cbReserved2 = 0;
   startupinfo.dwFlags = STARTF_USESTDHANDLES;
   startupinfo.hStdInput = hInputRead;
   startupinfo.hStdOutput = hOutputWrite;
   startupinfo.hStdError = hErrorWrite;
}
//---------------------------------------------------------------------------

bool __fastcall TMIXConsole::LaunchChildProcess(const AnsiString &cmdLine)
{
   if (!CreateProcess(
	  NULL,             // pointer to filename
	  cmdLine.c_str(),  // pointer to command line string
	  NULL, 		    // pointer to process security attributes
	  NULL, 		    // pointer to thread security attributes
	  true,  		    // handle inheritance flag
	  CREATE_NO_WINDOW, // creation flags
	  NULL, 		    // pointer to new environment block
	  NULL, 		    // pointer to current directory name
	 &startupinfo,      // pointer to STARTUPINFO
	 &processinfo       // pointer to PROCESS_INFORMATION
   ))
   {
	  ShowError("Error creating GPG process: \n");
	  return false;
   }

   return true;
}
//---------------------------------------------------------------------------

bool __fastcall TMIXConsole::SendPassphrase(char *passphrase)
{
//sm(passphrase);
   DWORD byteswrote;
   if (!WriteFile(hPassWrite, passphrase, strlen(passphrase), &byteswrote, NULL))
   {
	  ShowError("Error sending the passphrase to GPG: \n");
	  return false;
   }

   CLOSEHandle(hPassWrite);
   return true;
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::Terminate()
{
   if (running)
	  TerminateProcess(processinfo.hProcess, 0);
   terminated = true;
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::ShowError(const AnsiString &errormsg)
{
   MsgBox(Owner, errormsg + GetLastErrorStr(), MT_ERROR, MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::Log(const AnsiString &thread, int bytes)
{
   AnsiString count = bytes == -1 ? AnsiString("eof") : AnsiString(bytes);
   log->Add(AnsiString(GetTickCount())+"  "+thread+"  "+count);
}
//---------------------------------------------------------------------------

void __fastcall TMIXConsole::Log(const AnsiString &thread, const AnsiString &text)
{
   log->Add(AnsiString(GetTickCount())+"  "+thread+"  "+text);
}
//---------------------------------------------------------------------------

bool __fastcall TMIXConsole::Run(AnsiString cmdLine, char *Passphrase,
   TStream *Input, TStream *Output)
{
   inputstream = Input;
   outputstream = Output;

   if (!CreateHandles(cmdLine))
	  return false;

   SetupFileDescriptors(cmdLine);
   PrepareStartupInfo();

   if (!LaunchChildProcess(cmdLine))
	  return false;

   if (Passphrase && !SendPassphrase(Passphrase))
	  return false;

   running = true;
   try
   {
	  CLOSEHandle(hOutputWrite);
	  CLOSEHandle(hStatWrite);
	  CLOSEHandle(hErrorWrite);
	  if (inputstream)
	  {
		 inputthread = new TMIXStdIOInputThread(this);
	  }
	  outputthread = new TMIXStdIOOutputThread(this);
	  statusthread = new TMIXStdIOStatusThread(this);
	  errorthread = new TMIXStdIOErrorThread(this);

	  while (MsgWaitForMultipleObjects(1, &processinfo.hProcess, false,
		 INFINITE, QS_ALLINPUT) == WAIT_OBJECT_0 + 1)
			Application->ProcessMessages();
	  while (!statusthread->eof || !errorthread->eof) // || (inputthread && !inputthread->eof))
		 Application->ProcessMessages();
   }
   __finally
   {
	  if (!GetExitCodeProcess(processinfo.hProcess, &exitcode))
		 ShowError("Error getting GPG exit code: \n");

	  CloseHandle(processinfo.hThread);
	  CloseHandle(processinfo.hProcess);

	  status->LoadFromStream(statusstream);
	  error->LoadFromStream(errorstream);
	  delete errorthread;
	  delete statusthread;
	  delete outputthread;
	  if (inputthread)
	  {
		 if (!inputthread->eof)
			TerminateThread(inputthread, 0);
		 else
		    delete inputthread;
	  }
	  DestroyHandles();
	  running = false;
	  return true;
   }
}

//---------------------------------------------------------------------------
// TMIXStdIOInputThread
//---------------------------------------------------------------------------

__fastcall TMIXStdIOInputThread::TMIXStdIOInputThread(TMIXConsole *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TMIXStdIOInputThread::Execute()
{
   HANDLE objects[2];
   objects[0] = (void*)con->processinfo.hProcess;
   objects[1] = (void*)con->inputevent->Handle;
   con->inputevent->SetEvent();

   if (WaitForMultipleObjects(2, objects, false, INFINITE) == WAIT_OBJECT_0 + 1)
   {
	  char buffer[bufsize];
	  DWORD bytesWritten, bytesToWrite;
	  TStream *strm = con->inputstream;

	  strm->Seek(0, soFromBeginning);
	  while(strm->Position < strm->Size)
	  {
		 bytesToWrite = strm->Read(buffer, bufsize);
		 if (!WriteFile(con->hInputWrite, buffer, bytesToWrite, &bytesWritten, NULL))
		 {
			con->ShowError("Error sending input to GPG: \n");
			break;
		 }
		 if (con->terminated)
			break;
	  }
	  FlushFileBuffers(con->hInputWrite);
	  Application->ProcessMessages();
   }

   CLOSEHandle(con->hInputWrite);
   eof = true;
}

//---------------------------------------------------------------------------
// TMIXStdIOOutputThread
//---------------------------------------------------------------------------

__fastcall TMIXStdIOOutputThread::TMIXStdIOOutputThread(TMIXConsole *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TMIXStdIOOutputThread::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (!con->terminated && ReadFile(con->hOutputRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
   {
	  con->outputstream->Write(&buffer[0], bytesRead);
	  //if (logop) con->Log("o", bytesRead);
   }
   con->outputstream->Position = 0;
   //CLOSEHandle(con->hOutputRead);
   //if (logop) con->Log("o", "eof");
   eof = true;
}

//---------------------------------------------------------------------------
// TMIXStdIOStatusThread
//---------------------------------------------------------------------------

__fastcall TMIXStdIOStatusThread::TMIXStdIOStatusThread(TMIXConsole *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TMIXStdIOStatusThread::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (!con->terminated && ReadFile(con->hStatRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
   {
	  con->statusstream->Write(&buffer[0], bytesRead);
	  //if (logop) con->Log("s", bytesRead);
   }
   con->statusstream->Position = 0;
   //CLOSEHandle(con->hStatRead);
   //if (logop) con->Log("s", "eof");
   eof = true;
}

//---------------------------------------------------------------------------
// TMIXStdIOErrorThread
//---------------------------------------------------------------------------

__fastcall TMIXStdIOErrorThread::TMIXStdIOErrorThread(TMIXConsole *Console)
	: TThread(true)
{
   con = Console;
   Resume();
}
//---------------------------------------------------------------------------

void __fastcall TMIXStdIOErrorThread::Execute()
{
   DWORD bytesRead;
   char buffer[bufsize];

   while (!con->terminated && ReadFile(con->hErrorRead, buffer, bufsize, &bytesRead, NULL) && bytesRead > 0)
   {
	  con->errorstream->Write(&buffer[0], bytesRead);
	  //if (logop) con->Log("e", bytesRead);
   }
   con->errorstream->Position = 0;
   //CLOSEHandle(con->hErrorRead);
   //if (logop) con->Log("e", "eof");
   eof = true;
}
//---------------------------------------------------------------------------

AnsiString __fastcall getcommandline(const AnsiString &item, const AnsiString defvalue)
{
   TIniFile *ini = new TIniFile(AppIni());
   AnsiString cmdline = ini->ReadString("mix", item, defvalue);
   delete ini;
   return MixExe() + " " + cmdline.TrimLeft();
}
//---------------------------------------------------------------------------

bool __fastcall setfield(const AnsiString &field, const AnsiString &value,
   AnsiString &line)
{
   int x = line.Pos(field);
   if (x == 0)
	  return false;
   line.Delete(x, field.Length());
   line.Insert(value, x);
   return true;
}
//---------------------------------------------------------------------------

