//---------------------------------------------------------------------------
#ifndef mixconsoleH
#define mixconsoleH

#include <forms.hpp>
#include <syncobjs.hpp>
//---------------------------------------------------------------------------
class TMIXStdIOInputThread;
class TMIXStdIOOutputThread;
class TMIXStdIOStatusThread;
class TMIXStdIOErrorThread;
//---------------------------------------------------------------------------

int __fastcall MixExeMixMessage(TStrings *Message, AnsiString &Chain, int copies,
   bool ispost, TStrings *Feedback);

class TMIXConsole : public TObject
{
protected:
   TForm *Owner;
   bool running;
   STARTUPINFO startupinfo;

   void __fastcall PrepareStartupInfo();
   bool __fastcall CreateHandles(const AnsiString &cmdLine);
   void __fastcall SetupFileDescriptors(AnsiString &cmdline);
   bool __fastcall LaunchChildProcess(const AnsiString &cmdline);
   void __fastcall DestroyHandles();
public:
   bool terminated;
   HANDLE hOutputRead, hOutputWrite, hInputRead, hInputWrite, hPassRead,
	  hPassWrite, hErrorRead, hErrorWrite, hAttrRead, hAttrWrite,
	  hStatRead, hStatWrite;
   PROCESS_INFORMATION processinfo;
   TStream *inputstream, *outputstream;
   TMemoryStream *errorstream, *statusstream;
   TStringList *error, *status, *attrib, *log;
   TMIXStdIOOutputThread *outputthread;
   TMIXStdIOStatusThread *statusthread;
   TMIXStdIOErrorThread *errorthread;
   TMIXStdIOInputThread *inputthread;
   TEvent *inputevent;
   DWORD exitcode;

   __fastcall TMIXConsole(TForm *Owner);
   __fastcall virtual ~TMIXConsole();
   bool __fastcall Run(AnsiString cmdLine, char *Passphrase, TStream *Input, TStream *Output);
   void __fastcall Log(const AnsiString &thread, const AnsiString &text);
   void __fastcall Log(const AnsiString &thread, int bytes);
   void __fastcall ShowError(const AnsiString &errormsg);
   bool __fastcall SendPassphrase(char *passphrase);
   void __fastcall Terminate();
};

class TMIXStdIOInputThread : public TThread
{
protected:
	void __fastcall Execute();
public:
   bool eof;
   TMIXConsole *con;
   __fastcall TMIXStdIOInputThread(TMIXConsole *Console);
};

class TMIXStdIOOutputThread : public TThread
{
protected:
	void __fastcall Execute();
public:
   bool eof;
   TMIXConsole *con;
   __fastcall TMIXStdIOOutputThread(TMIXConsole *Console);
};

class TMIXStdIOStatusThread : public TThread
{
protected:
   void __fastcall Execute();
public:
   bool eof;
   TMIXConsole *con;
   __fastcall TMIXStdIOStatusThread(TMIXConsole *Console);
};

class TMIXStdIOErrorThread : public TThread
{
protected:
	void __fastcall Execute();
public:
   bool eof;
   TMIXConsole *con;
   __fastcall TMIXStdIOErrorThread(TMIXConsole *Console);
};


AnsiString MIXExeMixMessageCmd = "%msgtype%";

AnsiString MIXexemixmessage = "mixexemixmessage";

void __fastcall ShowMIXError(TForm *Owner, TStrings *Error);
AnsiString __fastcall getcommandline(const AnsiString &item, const AnsiString defvalue);
bool __fastcall setfield(const AnsiString &field, const AnsiString &value,
   AnsiString &line);
//---------------------------------------------------------------------------
#endif
