/*
*  packdbdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop
#include <stdio.h>

#include "packdbdlgs.h"
#include "global.h"
#include "utility.h"
#include "qsdbs.h"
#include "eraser.h"
#include "gpg.h"
#include "pgpactdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPackDBDlg *PackDBDlg;
AnsiString packmsg =
"\nPACKING THE QS DATABASE\n\n"
"There's seldom need to pack the QS database. It's true it never shrinks "
"(unless packed) but as data is deleted that space is used for new data before "
"the file is expanded.\n\nIf you find yourself in need of disk space and wish to "
"delete data from this database, you can then reclaim the disk space by packing "
"this file.";
//---------------------------------------------------------------------------

void __fastcall PackDatabase(TForm *Owner)
{
   PackDBDlg = new TPackDBDlg(Owner);
   PackDBDlg->ShowModal();
   delete PackDBDlg;
}
//---------------------------------------------------------------------------

__fastcall TPackDBDlg::TPackDBDlg(TComponent* Owner)
        : TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   Log->Clear();
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(Log->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   //AnsiString seteditrectspec = ini->ReadString(settings, "seteditrect", DefSetEditRectSpec);
   delete ini;

   // I set the editor position over panel1 with a 3 pixel margin on the left.
   // The goal is to make the text in the editor appear indented slightly.
   // SetEditRect(Editor) doesn't work in linux wine so I use this instead.
   // It didn't work quite right when I set it up in the designer. So...
   Log->Top = 0;
   Log->Left = RichEditIndent;
   Log->Height = Panel1->Height - 4;
   Log->Width = Panel1->Width - (RichEditIndent + 3);
   Log->Anchors = TAnchors() << akLeft << akTop << akRight << akBottom;
   Panel1->Color = Log->Color;
   //SetEditRect(Log);

   info->Caption = "";
   Log->Text = packmsg;
   itemfails = 0;
   fails = 0;
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::OkBtnClick(TObject *Sender)
{
   Log->Clear();
   Log->WordWrap = false;
   CancelBtn->Caption = "Stop";
   OkBtn->Enabled = false;
   Application->ProcessMessages();
   //SetEditRect(Log);
   PackDatabase();
   CancelBtn->Caption = "Close";
   if (cancel)
   {
      Log->Lines->Add("\noperation canceled!");
      info->Caption = "";
   }
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::CancelBtnClick(TObject *Sender)
{
   if (CancelBtn->Caption == "Close")
   {
      ModalResult = mrOk;
      return;
   }
   cancel = true;
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::LogLAST(const AnsiString &msg)
{
   Log->Lines->Strings[Log->Lines->Count-1] =
      Log->Lines->Strings[Log->Lines->Count-1] + msg;
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::LogADD(const AnsiString &msg)
{
   Log->Lines->Add(msg);
   Log->SelStart = Log->Text.Length();
   Log->Perform(EM_SCROLLCARET, 0, 0);
}

//---------------------------------------------------------------------------
// Pack Database
//---------------------------------------------------------------------------

bool __fastcall TPackDBDlg::PackItem(TDBindex *frIndex, TDBindex *toIndex,
   const AnsiString &key, int capfield)
{
   bool ok = true;
   AnsiString cap = GetTok(capfield, key, DBFSc);
   TDBbuffer *buf = new TDBbuffer();
   info->Caption = cap;

   if (frIndex->GetBinary(key, buf) != DB_OK)
   {
      LogADD("  Read Error: " + cap);
      ok = false;
      fails++;
   }

//if (key == SMTPHosts) sm(buf->data);

   if (ok && toIndex->PutBinary(key, buf, false) != DB_OK)
   {
      LogADD("  Write Error: " + cap);
      ok = false;
      fails++;
   }

   delete buf;
   return ok;
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::DoPackItem(TDBindex *frIndex, TDBindex *toIndex,
   const AnsiString &key, int capfield)
{
   Snooze(100);
   if (PackItem(frIndex, toIndex, key, capfield))
      LogLAST("ok");
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::PackIndex(TDBindex *frParent, TDBindex *toParent,
   const AnsiString &path, int capfield)
{
   itemfails = 0;
   TDBindex *frIndex = new TDBindex(frParent->dbm, path, frParent);
   TDBindex *toIndex = new TDBindex(toParent->dbm, path, toParent);

   for (int i = 0; i < frIndex->Count; i++)
   {
	  AnsiString key = frIndex->Strings[i];
	  if (frIndex->IsIndexKey(i))
	  {
		 LogADD("   " + key.LowerCase() + "...");
		 DoPackIndex(frIndex, toIndex, key, capfield);
	  }
	  else
	  {
		 if (!PackItem(frIndex, toIndex, key, capfield))
			itemfails++;
      }
      Application->ProcessMessages();
      if (cancel)
         break;
   }
   toIndex->Save();

   delete frIndex;
   delete toIndex;

   fails += itemfails;
}
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::DoPackIndex(TDBindex *frParent, TDBindex *toParent,
   const AnsiString &path, int capfield)
{
   Snooze(100);
   PackIndex(frParent, toParent, path, capfield);
   if (!itemfails)
      LogLAST("ok");
}
//---------------------------------------------------------------------------
// PackDatabase
//---------------------------------------------------------------------------

void __fastcall TPackDBDlg::PackDatabase()
{
   TQSDB *db = new TQSDB("");  // DB is InMemory
   assert(db != NULL);

   TDBindex *frMain, *toMain;
   frMain = DB->GetIndex(0);
   toMain = db->GetIndex(0);
   LogADD("packing database");

   LogADD("packing system...");
   DoPackItem(frMain, toMain, Config);
   if (cancel)
	  goto finish;

   LogADD("packing defaults...");
   DoPackIndex(frMain, toMain, Default);
   if (cancel)
	  goto finish;

   LogADD("packing contacts...");
   PackIndex(frMain, toMain, Contacts, 2);
   if (cancel)
	  goto finish;

   LogADD("packing drafts...");
   DoPackIndex(frMain, toMain, Drafts);
   if (cancel)
	  goto finish;

   LogADD("packing templates...");
   DoPackIndex(frMain, toMain, Templates);
   if (cancel)
	  goto finish;

   LogADD("packing mailboxes");
   PackIndex(frMain, toMain, Mailboxes, 2);
   if (cancel)
	  goto finish;

   LogADD("packing queues...");
   PackIndex(frMain, toMain, Queues, 2);
   if (cancel)
	  goto finish;

   LogADD("packing logs...");
   DoPackIndex(frMain, toMain, MailLogs);
   if (cancel)
	  goto finish;

   LogADD("packing notes...");
   DoPackIndex(frMain, toMain, Notes);
   if (cancel)
	  goto finish;

   LogADD("packing attachments...");
   DoPackIndex(frMain, toMain, Attachments);
   if (cancel)
	  goto finish;

   LogADD("packing smtp hosts...");
   DoPackItem(frMain, toMain, SMTPHosts);
   if (cancel)
	  goto finish;

   LogADD("packing pop3 hosts...");
   DoPackItem(frMain, toMain, POP3Hosts);
   if (cancel)
	  goto finish;

   LogADD("packing proxies...");
   DoPackItem(frMain, toMain, Proxies);
   if (cancel)
	  goto finish;

   LogADD("packing barred...");
   DoPackIndex(frMain, toMain, Bars);
   if (cancel)
	  goto finish;

   LogADD("packing session...");
   DoPackIndex(frMain, toMain, TheSession);
   if (cancel)
	  goto finish;

   if (DB->IsSecureMode())
   {
	  db->MasterPassword = new TMemoryStream();
	  db->MasterPassword->Write(DB->MasterPassword->Memory, DB->MasterPassword->Size);
	  ZeroMemory(DB->MasterPassword->Memory, DB->MasterPassword->Size);
	  delete DB->MasterPassword;
	  DB->MasterPassword = NULL;
   }

   toMain->Save();

   //LogADD("done");
   if (fails)
   {
	  LogADD(IntToStr(fails) + " items failed");
	  info->Caption = "";
   }
   else
   {
	  info->Caption = "Success!";
	  //LogADD("OK");
   }

   if (fails)
   {
	  if (MsgBox(this,
		 "You have " + IntToStr(fails) + " failed operations. \n"
		 "Do you wish to keep this new datafile?   ",
		  MT_CONFIRM, MB_YESNO) == IDNO)
	  {
		 goto finish;
	  }
   }

   if (DB->IsSecureMode())
   {
      if (DB->memstream)
         ZeroMemory(DB->memstream->Memory, DB->memstream->Size);
   }
   DB->modified = false;
   delete DB;
   if (FileExists(AppDBTmp()))
      eraserWipeFile(AppDBTmp());
   DB = db;
   db = NULL;
   DB->modified = true;
   DB->filename = AppDBTmp();
   LogADD("saving packed database...");
   if (DB->IsSecureMode())
   {
      AnsiString passphrase;
      GPGLoadMasterPassword(DB, passphrase);
      ShowEncryptingDatabase(DB, passphrase);
      DB->modified = false;
   }
   else
   {
      if (!DB->SaveToFile())
      {
         MsgBox(Screen->ActiveForm,
            "I failed to save packed database. I'm keeping the original!  \n"
            "The exception reported was: \n" +
			DB->exceptmsg,
            MT_ERROR, MB_OK);
      }
      else
      {
         Snooze(300);
	  }
   }
   LogLAST(FileExists(AppDBTmp()) ? "ok" : "FAILED");
   if (FileExists(AppDB()))
   {
      LogADD("wiping original...");
      eraserWipeFile(AppDB());
      Snooze(300);
      LogLAST(!FileExists(AppDB()) ? "ok" : "FAILED");
   }
   if (!FileExists(AppDB()))
      rename(AppDBTmp().c_str(), AppDB().c_str());
   DB->filename = AppDB();
   LogADD("done");
   if (FileExists(AppDB()))
      LogADD("OK");

finish:
   if (db)
   {
      delete db;
   }
}
//---------------------------------------------------------------------------

