/*
*  pgprecipsdlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "pgprecipsdlgs.h"
#include "pgpcpassdlgs.h"
//#include "pgp.h"
//#include "gpgman.h"
#include "global.h"
#include "utility.h"
//#include "appmsgs.h"
//#include "gpgerrdlgs.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPGPrecipsDlg *PGPrecipsDlg;
const int keypos = 2;
//---------------------------------------------------------------------------

bool __fastcall GetRecipientPGPPublicKeys(TForm *Owner, TStrings *msg,
   AnsiString &encryptkeys, bool &symmetric)
{
   PGPrecipsDlg = new TPGPrecipsDlg(Owner, msg);
   bool ok = PGPrecipsDlg->ShowModal() == mrOk;
   if (ok)
   {
      encryptkeys = PGPrecipsDlg->GetEncryptKeys();
      symmetric = PGPrecipsDlg->Symmetric->Checked;
   }
   delete PGPrecipsDlg;
   return ok;
}
//---------------------------------------------------------------------------

__fastcall TPGPrecipsDlg::TPGPrecipsDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TPGPrecipsDlg::TPGPrecipsDlg(TComponent* Owner, TStrings *Msg)
       : TForm(Owner)
{
   TIniFile *ini = new TIniFile(AppIni());
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(KeyList->Font, ini->ReadString("settings", "windowfont", OptsDefWindowFont));
   RecipList->Font->Assign(KeyList->Font);
   windowfontcolor = KeyList->Font->Color;
   //LoadListView("pgprecipdlg1", KeyList, ini);
   //LoadListView("pgprecipdlg2", RecipList,ini);
   delete ini;

   //algos = new TStringList();
   //ini = new TIniFile(GPGini());
   //ini->ReadSectionValues("algos", algos);
   //delete ini;

   msg = Msg;
   keys = new TStringList();
   msgman = new TMsgManager(msg);
}
//---------------------------------------------------------------------------

__fastcall TPGPrecipsDlg::~TPGPrecipsDlg()
{
   //SaveListView("pgprecipdlg1", KeyList);
   //SaveListView("pgprecipdlg2", RecipList);
   delete msgman;
   delete keys;
   //delete algos;
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (!Symmetric->Checked && !RecipList->Items->Count)
      {
         MsgBox(this,
            "You must either select at least one valid recipient   \n"
            "or choose the Conventional Encryption option.   \n",
             MT_INFO, MB_OK);
         CanClose = false;
         return;
      }

      if (Symmetric->Checked)
      {
         symmetrickey = GetCEncryptionPassword(this);
         if (symmetrickey.IsEmpty())
         {
            CanClose = false;
            return;
         }
         Snooze(100);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::ShowStatus(const AnsiString &statmsg)
{
 ((TForm*)Owner)->Perform(WM_SHOWSTATUS, 0, (LPARAM)statmsg.c_str());
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::LoadKeys()
{
   ShowStatus(" Loading keys...");
   TPGP *pgp = new TPGP(this);
   if (!pgp->CheckInitSuccess())
   {
      delete pgp;
      return;
   }

   pgp->ListPublicKeysWithColons(keys);
   delete pgp;

   InstallKeys();
   InstallRecips();
   ShowStatus("");
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::InstallKeys()
{
   KeyList->Items->BeginUpdate();
   KeyList->Items->Clear();
   for (int i = 1; i < keys->Count; i++)
   {
      AnsiString key = keys->Strings[i];
      if (AnsiString("ed").Pos(GetTok(1, key, ':')) > 0)
         continue;

      AddKey(KeyList, key);
   }
   KeyList->Items->EndUpdate();
   Label1->Caption = IntToStr(KeyList->Items->Count) + " user ids";
}
//---------------------------------------------------------------------------

inline void __fastcall TPGPrecipsDlg::InstallRecipsKey(TListItem *item, const AnsiString &recip,
   const AnsiString &key)
{
   assert(recip != "");
   assert(key != "");

   if (item == NULL)
      item = RecipList->Items->Add();

   AnsiString keyid, type, size;
   ParseKey(key, NULL, keyid, type, size);
   item->Caption = recip;
   item->SubItems->Add(keyid);
   item->SubItems->Add(type);
   item->SubItems->Add(size);
   item->SubItems->Add(key);
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::InstallRecips()
{
   AnsiString keyid, type, size;
   TStringList *recips = new TStringList();
   msgman->GetFullAddresses(recips);
   bool notgot = false;

   for (int i = 0; i < recips->Count; i++)
   {
      bool got = false;
      TListItem *item = RecipList->Items->Add();
      AnsiString key, recip = recips->Strings[i];
      for (int i = 0; i < keys->Count; i++)
      {
         key = keys->Strings[i];
         if (key.SubString(5, 1) == "e") // expired
            continue;
            
         if (key.Pos(recip) > 0)
         {
            InstallRecipsKey(item, recip, key);
            got = true;
            break;
         }
      }
      if (!got)
      {
         item->Caption = recip + " ";
         for (int i = 0; i < 4; i++)
            item->SubItems->Add("");
         notgot = true;
      }
   }

   if (notgot)
      Label2->Caption =
      "One or more recipient keys were not found. Please find the desired "
      "key from the list above or double-click item to remove.";

   delete recips;
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::ParseKey(const AnsiString &key, TStrings *uids,
   AnsiString &keyid, AnsiString &type, AnsiString &size)
{
   assert(key != "");

   /*AnsiString id, pak, salg, sbit, packet;
   int k = -1; while ((pak = GetTok(++k, key, cKS)) != "")
   {
      packet = GetTok(0, pak, ':');

      if (packet == "pub")
      {
         type = algos->Values[GetTok(3, pak, ':')];
         size = GetTok(2, pak, ':');
         id = GetTok(4, pak, ':');
         keyid = "0x" + id.SubString(id.Length() - 7, 8);
         if (uids)
            uids->Text = GetTok(9, pak, ':');
         continue;
      }

      if (!Primary->Checked && packet == "uid")
      {
         if (uids)
            uids->Add(GetTok(9, pak, ':'));
         continue;
      }

      if (packet == "sub")
      {
         salg = algos->Values[GetTok(3, pak, ':')];
         sbit = GetTok(2, pak, ':');
      }
   }
   if (!sbit.IsEmpty())
      size = sbit + "/" + size;
   if (!salg.IsEmpty())
      type += "/" + salg;*/
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::AddKey(TListView *list, const AnsiString &key)
{
   assert(list != NULL);
   assert(key != "");

   TListItem *item = list->Items->Add();
   item->Caption = GetTok(0, key, ':');
   item->SubItems->Add(GetTok(1, key, ':'));
   item->SubItems->Add(key);
}
//---------------------------------------------------------------------------

bool __fastcall TPGPrecipsDlg::KeyExpired(const AnsiString &key)
{
   assert(key.Length() > 4);
   return key[5] == 'e';
}
//---------------------------------------------------------------------------

bool __fastcall TPGPrecipsDlg::KeyExpired(TListView *list, int index)
{
   assert(list != NULL);
   assert(index >= 0);
   assert(index < list->Items->Count);
   return KeyExpired(GetKey(list, index));
}
//---------------------------------------------------------------------------

bool __fastcall TPGPrecipsDlg::KeyDisabled(const AnsiString &key)
{
   assert(key.Length() > 4);
   return key[5] == 'd';
}
//---------------------------------------------------------------------------

bool __fastcall TPGPrecipsDlg::KeyDisabled(TListView *list, int index)
{
   assert(list != NULL);
   assert(index >= 0);
   assert(index < list->Items->Count);
   return KeyDisabled(GetKey(list, index));
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPrecipsDlg::GetKey(TListView *list, int index)
{
   assert(list != NULL);
   assert(index >= 0);
   assert(index < list->Items->Count);
   TListItem *item = list->Items->Item[index];
//sm(item->Caption +IntToStr(item->SubItems->Count));
   assert(item->SubItems->Count == keypos + 1);
   return item->SubItems->Strings[keypos];
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPrecipsDlg::GetKeyID(TListView *list, int index)
{
   assert(list != NULL);
   assert(index >= 0);
   assert(index < list->Items->Count);
   TListItem *item = list->Items->Item[index];
   assert(item->SubItems->Count == keypos + 1);
   return item->SubItems->Strings[0];
}
//---------------------------------------------------------------------------

bool __fastcall TPGPrecipsDlg::KeyExists(TListView *list, const AnsiString &key)
{
   assert(list != NULL);
   assert(key != "");

   bool exist = false;
   for (int i = 0; i < list->Items->Count; i++)
   {
      if (GetKey(list, i) == key)
      {
         exist = true;
         break;
      }
   }
   return exist;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPrecipsDlg::GetListKey(TListView *list, int index)
{
   assert(list != NULL);
   assert(index >= 0);
   assert(index < list->Items->Count);
   AnsiString userid = list->Items->Item[index]->Caption;
   return userid + KS + list->Items->Item[index]->SubItems->Strings[keypos];
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::FormShow(TObject *Sender)
{
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   LoadKeys();
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::SymmetricClick(TObject *Sender)
{
   if (Symmetric->Checked)
   {
      KeyList->Items->BeginUpdate();
      KeyList->ParentColor = true;
      KeyList->Items->Clear();
      KeyList->Enabled = false;
      KeyList->Items->EndUpdate();

      RecipList->Items->BeginUpdate();
      RecipList->ParentColor = true;
      RecipList->Items->Clear();
      RecipList->Enabled = false;
      RecipList->Items->EndUpdate();

      Primary->Enabled = false;
      Label1->Enabled = false;
      Label2->Enabled = false;
      usermsg = Label2->Caption;
      //Label2->Caption = "Click Ok to enter Conventional Encryption passphrase.";
      Snooze(200);
      OkBtn->Click();

   }
   else
   {
      SetRedraw(this, 0);
      KeyList->Items->BeginUpdate();
      KeyList->Color = clWindow;
      InstallKeys();
      KeyList->Enabled = true;
      KeyList->Items->EndUpdate();

      RecipList->Items->BeginUpdate();
      RecipList->Color = clWindow;
      InstallRecips();
      RecipList->Enabled = true;
      RecipList->Items->EndUpdate();
      SetRedraw(this, 1);

      Primary->Enabled = true;
      Label1->Enabled = true;
      //Label2->Enabled = true;
      Label2->Caption = usermsg;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::KeyListCustomDrawItem(
      TCustomListView *Sender, TListItem *Item, TCustomDrawState State,
      bool &DefaultDraw)
{
   if (KeyList->Items->Count == 1 && KeyList->Items->Item[0]->Caption == "Loading...")
      return;

   TCanvas *canvas = Sender->Canvas;
   if (KeyExpired(KeyList, Item->Index) || KeyDisabled(KeyList, Item->Index))
   {
      canvas->Font->Style = TFontStyles() << fsItalic;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::RecipListCustomDrawItem(
      TCustomListView *Sender, TListItem *Item, TCustomDrawState State,
      bool &DefaultDraw)
{
   TCanvas *canvas = Sender->Canvas;
   assert(Item->SubItems->Count == keypos + 1);
   if (Item->SubItems->Strings[0].IsEmpty())
   {
      canvas->Font->Style = TFontStyles() << fsStrikeOut;
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::RecipListDblClick(TObject *Sender)
{
   assert(RecipList->Selected);
   RecipList->Selected->Delete();
}
//---------------------------------------------------------------------------

AnsiString __fastcall TPGPrecipsDlg::GetEncryptKeys()
{
   if (Symmetric->Checked)
   {
      return symmetrickey;
   }

   AnsiString key, keys;
   for (int i = 0; i < RecipList->Items->Count; i++)
   {
      key = GetKeyID(RecipList, i);
      if (!key.IsEmpty())
      {
         if (keys.Length())
            keys += ",";
         keys += key;
      }
   }
   return keys;
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::KeyListDblClick(TObject *Sender)
{
   assert(KeyList->Selected);
   AnsiString userid, key, keystr = GetListKey(KeyList, KeyList->Selected->Index);
   userid = GetTok(0, keystr, cKS);
   key = GetTok(1, keystr, cKS);

   if (!KeyExists(RecipList, key) && !KeyExpired(key) && !KeyDisabled(key))
      InstallRecipsKey(NULL, userid, key);
   else
      MessageBeep(0);
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::RecipListDragDrop(TObject *Sender,
      TObject *Source, int X, int Y)
{
   TListView *src = dynamic_cast<TListView*>(Source);
   assert(src != NULL);

   if (src == KeyList)
   {
      AnsiString listkey = GetListKey(KeyList, KeyList->Selected->Index);
      if (!KeyExists(RecipList, GetTok(1, listkey, cKS)))
         InstallRecipsKey(NULL, GetTok(0, listkey, cKS), GetTok(1, listkey, cKS));
      else
         MessageBeep(0);
   }
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::RecipListDragOver(TObject *Sender,
      TObject *Source, int X, int Y, TDragState State, bool &Accept)
{
   TListView *src = dynamic_cast<TListView*>(Source);
   assert(src != NULL);

   Accept = false;
   assert(KeyList->Selected->SubItems->Count == keypos + 1);
   AnsiString key = KeyList->Selected->SubItems->Strings[keypos];
   if (src == KeyList && !KeyExpired(key) && !KeyDisabled(key))
      Accept = true;
}
//---------------------------------------------------------------------------

void __fastcall TPGPrecipsDlg::PrimaryClick(TObject *Sender)
{
   InstallKeys();
}
//---------------------------------------------------------------------------

