/*
*  proxymandlgs.cpp 
*
*  Copyright (C) 2010 Richard Christman 
*
*  This file is part of QuickSilver Lite.
*
*  QuickSilver Lite is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by the 
*  Free Software Foundation; either version 3 of the License, or (at your 
*  option) any later version.
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
*  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
*  for more details.
* 
*  You should have received a copy of the GNU General Public License along 
*  with this program; if not, see <http://www.gnu.org/licenses>.
* 
*  Additional permission under GNU GPL version 3 section 7
* 
*  If you modify QuickSilver Lite, or any covered work, by linking or 
*  combining it with CodeGear VCL (or a modified version of that library), 
*  containing parts covered by the terms of CodeGear's license, the licensors 
*  of QuickSilver Lite grant you additional permission to convey the 
*  resulting work.
*
*/

//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "proxymandlgs.h"
#include "proxydlgs2.h"
#include "utility.h"
#include "global.h"
#include "qsdbs.h"
#include "composewins.h"
#include "help.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TProxyManDlg *ProxyManDlg;
//---------------------------------------------------------------------------

bool __fastcall ManageProxies(TForm *Owner)
{
   TProxyManDlg *proxyman = new TProxyManDlg(Owner);
   proxyman->ShowModal();
   bool ok = proxyman->ModalResult == mrOk;
   delete proxyman;
   return ok;

}
//---------------------------------------------------------------------------

bool __fastcall LoadProxyBoxHTTP(TComboBox *ProxyBox, AnsiString &defhttp)
{
   TStringList *proxies = new TStringList();
   TIniFile *ini = new TIniFile(AppIni());
   defhttp = ini->ReadString("settings", "httpproxy", "none");
   DB->GetProxyNames(proxies, NULL, NULL, NULL);

   ProxyBox->Items->Clear();
   ProxyBox->Sorted = true;
   if (proxies->Count)
      ProxyBox->Items->AddStrings(proxies);
   ProxyBox->Sorted = false;
   ProxyBox->Items->Insert(0, "none");

   ProxyBox->ItemIndex = ProxyBox->Items->IndexOf(defhttp);
   if (ProxyBox->ItemIndex < 0)
   {
      MsgBox(Screen->ActiveForm,
         "The HTTP proxy you've been using, " + defhttp + ", is missing! You  \n"
         "don't currently have a proxy assigned!",
		  MT_WARN, MB_OK);
	  defhttp = "none";
	  ini->WriteString("settings", "httpproxy", defhttp);
	  ProxyBox->ItemIndex = 0;
   }

   delete proxies;
   delete ini;
   return ProxyBox->Items->Count > 0;
}
//---------------------------------------------------------------------------

bool __fastcall LoadProxyBoxSMTP(TComboBox *ProxyBox, AnsiString &defsmtp)
{
   TStringList *proxies = new TStringList();
   TIniFile *ini = new TIniFile(AppIni());
   defsmtp = ini->ReadString("settings", "smtpproxy", "none");
   DB->GetProxyNames(NULL, proxies, NULL, NULL);

   ProxyBox->Items->Clear();
   ProxyBox->Sorted = true;
   if (proxies->Count)
	  ProxyBox->Items->AddStrings(proxies);
   ProxyBox->Sorted = false;
   ProxyBox->Items->Insert(0, "none");

   ProxyBox->ItemIndex = ProxyBox->Items->IndexOf(defsmtp);
   if (ProxyBox->ItemIndex < 0)
   {
	  MsgBox(Screen->ActiveForm,
		 "The SMTP proxy you've been using, " + defsmtp + ", is missing! You  \n"
		 "don't currently have a proxy assigned!",
		  MT_WARN, MB_OK);
	  defsmtp = "none";
	  ini->WriteString("settings", "smtpproxy", defsmtp);
	  ProxyBox->ItemIndex = 0;
   }

   delete proxies;
   delete ini;
   return ProxyBox->Items->Count > 1;
}
//---------------------------------------------------------------------------

__fastcall TProxyManDlg::TProxyManDlg(TComponent* Owner)
		: TForm(Owner)
{
   PopupParent = (TForm*)Owner;
   TIniFile *ini = new TIniFile(AppIni());
   LoadWindowPosition(this, "proxymandlg", ListView, ini);
   orgwin = IntToStr(Width)+IntToStr(Height) +
      IntToStr(ListView->Columns->Items[0]->Width) +
      IntToStr(ListView->Columns->Items[1]->Width) +
      IntToStr(ListView->Columns->Items[2]->Width) +
      IntToStr(ListView->Columns->Items[3]->Width);
   AssignFont(Font, ini->ReadString("settings", "dialogfont", OptsDefDialogFont));
   AssignFont(ListView->Font, ini->ReadString("settings", "windowfont",
      OptsDefWindowFont));
   defsmtp = ini->ReadString("settings", "smtpproxy", "none");
   defhttp = ini->ReadString("settings", "httpproxy", "none");
   delete ini;

   TStringList *proxies = new TStringList();
   DB->LoadProxies(proxies);
   SetProxies(proxies);
   delete proxies;

   LoadSelectorBoxes();
   Timer1->Enabled = true;
}
//---------------------------------------------------------------------------

__fastcall TProxyManDlg::~TProxyManDlg()
{
   if (IntToStr(Width)+IntToStr(Height)+
       IntToStr(ListView->Column[0]->Width)+
       IntToStr(ListView->Column[1]->Width)+
       IntToStr(ListView->Column[2]->Width)+
       IntToStr(ListView->Column[3]->Width) != orgwin)
   {
      TIniFile *ini = new TIniFile(AppIni());
      SaveWindowPosition(this, "proxymandlg", ListView, ini);
      delete ini;
   }

   if (defsmtpchanged || defhttpchanged)
      Application->MainForm->Perform(WM_APPINICHANGE, 0, 0);
}
//---------------------------------------------------------------------------

//void __fastcall TProxyManDlg::LoadSelectorBoxes()
//{
//   LoadProxyBoxSMTP(SMTPBox, defsmtp);
//   LoadProxyBoxHTTP(HTTPBox, defhttp);
//}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::LoadSelectorBoxes()
{
   TStringList *proxy = new TStringList();
   TStringList *proxies = new TStringList();

   SMTPBox->Items->Clear();
   SMTPBox->Sorted = true;
   HTTPBox->Items->Clear();
   HTTPBox->Sorted = true;
   GetProxies(proxies);
   for (int i = 0; i < proxies->Count; i++)
   {
	  AnsiString name = proxies->Names[i];
	  proxy->CommaText = proxies->Values[name];
	  if (proxy->Values["smtpproxy"] == "1")
		 SMTPBox->Items->Add(name);
	  if (proxy->Values["httpproxy"] == "1")
		 HTTPBox->Items->Add(name);
   }
   HTTPBox->Sorted = false;
   SMTPBox->Sorted = false;

   HTTPBox->Items->Insert(0, "none");
   SMTPBox->Items->Insert(0, "none");
   SMTPBox->ItemIndex = SMTPBox->Items->IndexOf(defsmtp);
   HTTPBox->ItemIndex = HTTPBox->Items->IndexOf(defhttp);
   CheckReportDefaultProxyMissing();

   ProxyBoxSelect(SMTPBox);
   ProxyBoxSelect(HTTPBox);
   
   delete proxies;
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::CheckReportDefaultProxyMissing()
{
   if (SMTPBox->ItemIndex < 0)
   {
	  MsgBox(this,
		 "The SMTP proxy you've been using, " + defsmtp + ", is missing! You  \n"
		 "don't currently have a proxy assigned.",
		 MT_WARN, MB_OK);
	  SMTPBox->ItemIndex = 0;
      defsmtp = "none";
      TIniFile *ini = new TIniFile(AppIni());
      ini->WriteString("settings", "smtpproxy", defsmtp);
      delete ini;
   }
   if (HTTPBox->ItemIndex < 0)
   {
      MsgBox(this,
         "The HTTP proxy you've been using, " + defhttp + ", is missing! You  \n"
         "don't currently have a proxy assigned.",
         MT_WARN, MB_OK);
      HTTPBox->ItemIndex = 0;
      defhttp = "none";
      TIniFile *ini = new TIniFile(AppIni());
      ini->WriteString("settings", "httpproxy", defhttp);
      delete ini;
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::SetProxies(TStrings *proxies)
{
   TStringList *proxy = new TStringList();

   ListView->Items->Clear();
   ListView->Items->BeginUpdate();
   for (int i = 0; i < proxies->Count; i++)
   {
      proxy->CommaText = proxies->Values[proxies->Names[i]];
      TListItem *item = ListView->Items->Add();
      SetProxy(item, proxy);
   }
   ListView->Items->EndUpdate();

   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::GetProxy(TListItem *item, TStrings *proxy)
{
   assert(item != NULL);
   assert(proxy != NULL);
   proxy->Clear();
   proxy->Values["proxy"] = item->Caption;
   proxy->Values["host"] = item->SubItems->Strings[0];
   proxy->Values["socks"] = item->SubItems->Strings[2];
   proxy->Values["tor"] = item->SubItems->Strings[3] == "*" ? "1" : "0";
   proxy->Values["smtpproxy"] = item->SubItems->Strings[4];
   proxy->Values["smtpport"] = item->SubItems->Strings[5];
   proxy->Values["httpproxy"] = item->SubItems->Strings[6];
   proxy->Values["httpport"] = item->SubItems->Strings[7];
   proxy->Values["auth"] = item->SubItems->Strings[8];
   proxy->Values["userid"] = item->SubItems->Strings[9];
   proxy->Values["password"] = item->SubItems->Strings[10];
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::SetProxy(TListItem *item, TStrings *proxy)
{
   AnsiString proto;
   if (proxy->Values["smtpproxy"] == "1")
      proto += "S";
   if (proxy->Values["httpproxy"] == "1")
      proto += "H";
   while (item->SubItems->Count < 11)
      item->SubItems->Add("");
   item->Caption = proxy->Values["proxy"];
   item->SubItems->Strings[0] = proxy->Values["host"];
   item->SubItems->Strings[1] = proto;
   item->SubItems->Strings[2] = proxy->Values["socks"];
   item->SubItems->Strings[3] = proxy->Values["tor"] == "1" ? "*" : "";
   item->SubItems->Strings[4] = proxy->Values["smtpproxy"];
   item->SubItems->Strings[5] = proxy->Values["smtpport"];
   item->SubItems->Strings[6] = proxy->Values["httpproxy"];
   item->SubItems->Strings[7] = proxy->Values["httpport"];
   item->SubItems->Strings[8] = proxy->Values["auth"];
   item->SubItems->Strings[9] = proxy->Values["userid"];
   item->SubItems->Strings[10] = proxy->Values["password"];
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::GetProxies(TStrings *proxies)
{
   TStringList *proxy = new TStringList();

   proxies->Clear();
   for (int i = 0; i < ListView->Items->Count; i++)
   {
      TListItem *item = ListView->Items->Item[i];
      GetProxy(item, proxy);
      proxies->Values[item->Caption] = proxy->CommaText;
   }
   ListView->Items->EndUpdate();

   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::PopupMenu1Popup(TObject *Sender)
{
   Edit1->Enabled = ListView->Selected;
   Delete1->Enabled = ListView->Selected;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Resizedialog1Click(TObject *Sender)
{
   BorderStyle = bsSizeable;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Locksize1Click(TObject *Sender)
{
   BorderStyle = bsDialog;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
   if (modified)
   {
      if (ModalResult == mrCancel)
      {
         switch (MsgBox(this,
            "Your proxy info has been modified. If you cancel you'll lose all  \n"
            "your changes. Save your changes? ",
             MT_CONFIRM, MB_YESNOCANCEL))
         {
            case IDCANCEL: CanClose = false;
            case IDNO: return;
         }
      }

      TStringList *proxies = new TStringList();
      GetProxies(proxies);
      DB->SaveProxies(proxies);
      delete proxies;
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Timer1Timer(TObject *Sender)
{
   Timer1->Enabled = false;
   CheckReportDefaultProxyMissing();
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::SMTPBoxChange(TObject *Sender)
{
   assert(SMTPBox->Text != "");
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "smtpproxy", SMTPBox->Text);
   defsmtp = SMTPBox->Text;
   defsmtpchanged = true;
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Add1Click(TObject *Sender)
{
   TStringList *proxy = new TStringList();
   TStringList *proxies = new TStringList();

   GetProxies(proxies);
   if (NewProxy(this, proxy, proxies))
   {
      TListItem *item = ListView->Items->Add();
      SetProxy(item, proxy);
      LoadSelectorBoxes();
      modified = true;
   }

   delete proxies;
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Edit1Click(TObject *Sender)
{
   assert(ListView->Selected != NULL);
   TStringList *proxy = new TStringList();
   TStringList *proxies = new TStringList();

   GetProxies(proxies);
   GetProxy(ListView->Selected, proxy);
   AnsiString newname, oldname = proxy->Values["proxy"];
   if (EditProxy(this, proxy, proxies))
   {
      newname = proxy->Values["proxy"];
      if (newname != oldname)
      {
         if (SMTPBox->Text == oldname)
         {
            SMTPBox->Items->Delete(SMTPBox->ItemIndex);
            SMTPBox->ItemIndex = SMTPBox->Items->Add(newname);
            SMTPBoxChange(NULL);
         }
         if (HTTPBox->Text == oldname)
         {
            HTTPBox->Items->Delete(HTTPBox->ItemIndex);
			HTTPBox->ItemIndex = HTTPBox->Items->Add(newname);
            HTTPBoxChange(NULL);
         }
      }
      SetProxy(ListView->Selected, proxy);
      modified = true;
   }
   delete proxies;
   delete proxy;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::Delete1Click(TObject *Sender)
{
   assert(ListView->Selected != NULL);
   AnsiString name = ListView->Selected->Caption;
   if (MsgBox(this,
      "Delete the proxy, " + name + "?  \n",
       MT_CONFIRM, MB_OKCANCEL) == IDOK)
   {
	  ListView->Selected->Delete();
	  DeleteBtn->Enabled = false;
	  EditBtn->Enabled = false;
      modified = true;
      int x = SMTPBox->Items->IndexOf(name);
      if (x >= 0)
      {
         if (SMTPBox->Text == name)
         {
            SMTPBox->ItemIndex = 0;
            TIniFile *ini = new TIniFile(AppIni());
            ini->WriteString("settings", "smtpproxy", "none");
            delete ini; Snooze(200);
            MsgBox(this,
              "That was you default SMTP proxy. Now, there's none assigned.  ",
               MT_WARN, MB_OK);
            defsmtp = "none";
         }
         SMTPBox->Items->Delete(x);
      }
      x = HTTPBox->Items->IndexOf(name);
      if (x >= 0)
      {
         if (HTTPBox->Text == name)
         {
            HTTPBox->ItemIndex = 0;
			TIniFile *ini = new TIniFile(AppIni());
			ini->WriteString("settings", "httpproxy", "none");
			delete ini; Snooze(200);
			MsgBox(this,
			  "That was you default HTTP proxy. Now, there's none assigned.  ",
			   MT_WARN, MB_OK);
			defhttp = "none";
		 }
		 HTTPBox->Items->Delete(x);
	  }
   }
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::HelpBtnClick(TObject *Sender)
{
   DisplayHtmlHelpTopic(this, "::/html/proxyman.htm>qscshelp");
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ListViewClick(TObject *Sender)
{
   EditBtn->Enabled = ListView->Selected;
   DeleteBtn->Enabled = EditBtn->Enabled;
}
//---------------------------------------------------------------------------
// deprecated: this function does not work when using ProxyBoxSelect below
void __fastcall TProxyManDlg::HTTPBoxChange(TObject *Sender)
{
   assert(HTTPBox->Text != "");
   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", "httpproxy", HTTPBox->Text);
   defhttp = HTTPBox->Text;
   defhttpchanged = true;
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ProxyBoxSelect(TObject *Sender)
{
   TComboBox *ProxyBox = dynamic_cast<TComboBox*>(Sender);
   if (!ProxyBox) return;

   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
   else
	 ProxyBox->Font->Style = TFontStyles();

   AnsiString theproxy;
   if (ProxyBox == SMTPBox)
   {
	  defsmtp = ProxyBox->Text;
	  theproxy = "smtpproxy";
   }
   else
   {
	  defhttp = ProxyBox->Text;
	  theproxy = "httpproxy";
   }

   TIniFile *ini = new TIniFile(AppIni());
   ini->WriteString("settings", theproxy, ProxyBox->Text);
   delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ProxyBoxDropDown(TObject *Sender)
{
   TComboBox *ProxyBox = dynamic_cast<TComboBox*>(Sender);
   if (!ProxyBox) return;

   ProxyBox->Font->Style = TFontStyles();
}
//---------------------------------------------------------------------------

void __fastcall TProxyManDlg::ProxyBoxCloseUp(TObject *Sender)
{
   TComboBox *ProxyBox = dynamic_cast<TComboBox*>(Sender);
   if (!ProxyBox) return;

   if (ProxyBox->Text == "none")
	 ProxyBox->Font->Style = TFontStyles() << fsBold;
}
//---------------------------------------------------------------------------

